# -*- coding: utf-8 -*-

from plot import *

minfloat = 4.9406564584124654e-324
maxfloat = 8.9884656743115795e+307

import matplotlib as mpl
mpl.rcParams['lines.linewidth'] = 2
mpl.rcParams['lines.markeredgewidth'] = 2
mpl.rcParams['lines.markersize'] = 8
mpl.rcParams['font.size'] = 18
mpl.rcParams['legend.fontsize'] = 18

markers = ['o', '|', '+', 's', 'x', '*']

# error norms

def L1(data, refd=None):
    if refd is None:
        n = self.f.MAXX*self.f.MAXY*reduce((lambda x,y:x*y), data.shape)
        return abs(data).sum()/n
    else:
        return abs(data - refd).sum()/abs(refd).sum()

def L2(data, refd=None):
    if refd is None:
        n = self.f.MAXX*self.f.MAXY*reduce((lambda x,y:x*y), data.shape)
        return (numpy.sqr(data).sum()/n)**0.5
    else:
        return ((data - refd)**2).sum()**0.5/((refd**2).sum()**0.5)

def Linf(data, refd=None):
    if refd is None:
        return numpy.max(abs(data))
    else:
        return numpy.max(abs(data - refd))/numpy.max(abs(refd))

# plotting routines

def plotError ( name, v, N, case='mean', frame=False, color=None, refr=None, K=1, infolines=False, save_suffix='', path='', label_suffix='', order=1, marker='x' ):
  '''Plots the errors for the refinements in the list N'''
  
  reft = len(refr.times)-1
  refd = refr.var(v, reft, case)
  
  errors = []
  ncells = []
  rtimes = []
  
  # error estimates of the MC quadratures for the L^2-error
  errors_K = []
  
  for i , (NX, NY, NZ) in enumerate(N):
    
    refdd = refd
    
    zoom = refr.i.NX/NX
    if refr.i.NY == 1:
      while zoom > 1:
        refdd = (refdd[0:refr.i.NX-1:2,:,:]+refdd[1:refr.i.NX:2,:,:])/2
        zoom /= 2
    elif refr.i.NZ == 1:
      while zoom > 1:
        refdd = (refdd[0:refr.i.NX-1:2,0:refr.i.NY-1:2,:]+refdd[1:refr.i.NX:2,0:refr.i.NY-1:2,:]+refdd[1:refr.i.NX:2,1:refr.i.NY:2,:]+refdd[0:refr.i.NX-1:2,1:refr.i.NY:2,:])/4
        zoom /= 2
    else:
      while zoom > 1:
        refdd = (  refdd[0:refr.i.NX-1:2, 0:refr.i.NY-1:2, 0:refr.i.NZ-1:2] \
                 + refdd[0:refr.i.NX-1:2, 0:refr.i.NY-1:2, 1:refr.i.NZ:2  ] \
                 + refdd[0:refr.i.NX-1:2, 1:refr.i.NY:2,   0:refr.i.NZ-1:2] \
                 + refdd[0:refr.i.NX-1:2, 1:refr.i.NY:2,   1:refr.i.NZ:2  ] \
                 + refdd[1:refr.i.NX:2,   0:refr.i.NY-1:2, 0:refr.i.NZ-1:2] \
                 + refdd[1:refr.i.NX:2,   0:refr.i.NY-1:2, 1:refr.i.NZ:2  ] \
                 + refdd[1:refr.i.NX:2,   1:refr.i.NY:2,   0:refr.i.NZ-1:2] \
                 + refdd[1:refr.i.NX:2,   1:refr.i.NY:2,   1:refr.i.NZ:2  ] ) / 8
        zoom /= 2
    
    name_base = name + '_%dx%dx%d' % (NX, NY, NZ)
    
    error = 0
    rtime = 0
    mean_K = 0
    M2_K = 0
    for k in range(1, K+1):
      r = Run('%s-%d' % (name_base, k), path=path)
      
      t = len(r.times)-1
      d = r.var(v, t, case)
      
      if order == 1:
        rel_err = (100*L1(d, refdd))**2
      elif order == 2:
        rel_err = (100*L2(d, refdd))**2
      
      error += rel_err
      delta = rel_err - mean_K
      mean_K += delta / k
      if k > 1:
        M2_K += delta * (rel_err - mean_K)
      
      if 'wallclock_runtime_global' in r.runtime:
        rtime += r.runtime['wallclock_runtime_global']
      elif 'wall_runtimeclock_total' in r.runtime:
        rtime += r.runtime['wallclock_runtime_total'] # legacy
      else:
        rtime += r.runtime['clock_runtime']
    
    errors.append ( numpy.sqrt(error/K) )
    ncells.append ( NX )
    rtimes.append ( rtime/K )
    variance_K  = M2_K / max(1,K-1)
    rel_error_K = numpy.sqrt(variance_K / K) / (error/K)
    errors_K.append ( rel_error_K )
  
  # plot error vs ncells
  
  pylab.subplot(121)
  if K == 1:
	pylab.plot(numpy.log10(ncells), numpy.log10(errors), '-' + marker, label=name+label_suffix, color=color)
  elif errors_K[-1] < 0.01:
	pylab.plot(numpy.log10(ncells), numpy.log10(errors), '-' + marker, label=name+label_suffix+r' ($\sigma_K$<0.01)', color=color)
  else:
    pylab.plot(numpy.log10(ncells), numpy.log10(errors), '-' + marker, label=name+label_suffix+r' ($\sigma_K$'+'=%.2f)' % errors_K[-1], color=color)
  pylab.title(r'Relative $L^%d$-error of ' % order + case + ' of ' + v.name + ' (K=%d)' % K)
  pylab.ylabel(r'log10( $L^2(\Omega,\ L^%d)$ error )' % order)
  pylab.xlabel('log10(cells) in x-direction')
  pylab.legend(loc='lower left')
  pylab.grid(True)
  
  slopes = [[1,2], [1,1]]
  if not frame:
    for slope in slopes:
      plotSlope(slope, pylab.gca())
  
  # plot error vs runtime
  
  pylab.subplot(122)
  pylab.plot(numpy.log10(rtimes), numpy.log10(errors), '-' + marker, label=name+label_suffix, color=color)
  pylab.title(r'Relative $L^%d$-error of ' % order + case + ' of ' + v.name + ' (K=%d)' % K)
  pylab.ylabel(r'log10( $L^2(\Omega,\ L^%d)$ error )' % order)
  pylab.xlabel('log10(seconds)')
  pylab.legend(loc='lower left')
  pylab.grid(True)
  
  dim = int(refr.i.NX>1) + int(refr.i.NY>1) + int(refr.i.NZ>1)
  
  slopes = [[1,2+2*dim], [1,1+dim]]
  if not frame:
    for slope in slopes:
      plotSlope(slope, pylab.gca())
  
  pylab.subplots_adjust(right=0.98)
  pylab.subplots_adjust(left=0.07)
  pylab.subplots_adjust(top=0.92)
  
  if infolines:
    refr.showInfo('Reference ')
    pylab.subplots_adjust(bottom=0.25)
  else:
    pylab.subplots_adjust(bottom=0.12)
  
  if not frame:
    save = getFileName(refr, 'conv-' + case + save_suffix)
    pylab.savefig(save)
    refr.generateTexTable(save)
    pylab.show()

def plotErrorVsCells ( name, v, N, case='mean', frame=False, color=None, refr=None, K=1, infolines=False, save_suffix='', path='', label_suffix='', marker='x' ):
  '''Plots the errors for the refinements in the list N vs number of cells'''
  
  reft = len(refr.times)-1
  refd = refr.var(v, reft, case)
  
  errors = []
  ncells = []
  rtimes = []
  
  for i , (NX, NY, NZ) in enumerate(N):
    
    refdd = refd
    
    zoom = refr.i.NX/NX
    if refr.i.NY == 1:
      while zoom > 1:
        refdd = (refdd[0:refr.i.NX-1:2,:,:]+refdd[1:refr.i.NX:2,:,:])/2
        zoom /= 2
    elif refr.i.NZ == 1:
      while zoom > 1:
        refdd = (refdd[0:refr.i.NX-1:2,0:refr.i.NY-1:2,:]+refdd[1:refr.i.NX:2,0:refr.i.NY-1:2,:]+refdd[1:refr.i.NX:2,1:refr.i.NY:2,:]+refdd[0:refr.i.NX-1:2,1:refr.i.NY:2,:])/4
        zoom /= 2
    else:
      while zoom > 1:
        refdd = (  refdd[0:refr.i.NX-1:2, 0:refr.i.NY-1:2, 0:refr.i.NZ-1:2] \
                 + refdd[0:refr.i.NX-1:2, 0:refr.i.NY-1:2, 1:refr.i.NZ:2  ] \
                 + refdd[0:refr.i.NX-1:2, 1:refr.i.NY:2,   0:refr.i.NZ-1:2] \
                 + refdd[0:refr.i.NX-1:2, 1:refr.i.NY:2,   1:refr.i.NZ:2  ] \
                 + refdd[1:refr.i.NX:2,   0:refr.i.NY-1:2, 0:refr.i.NZ-1:2] \
                 + refdd[1:refr.i.NX:2,   0:refr.i.NY-1:2, 1:refr.i.NZ:2  ] \
                 + refdd[1:refr.i.NX:2,   1:refr.i.NY:2,   0:refr.i.NZ-1:2] \
                 + refdd[1:refr.i.NX:2,   1:refr.i.NY:2,   1:refr.i.NZ:2  ] ) / 8
        zoom /= 2
    
    name_base = name + '_%dx%dx%d' % (NX, NY, NZ)
    
    error = 0
    rtime = 0
    for k in range(1, K+1):
      r = Run('%s-%d' % (name_base, k), path=path)
      
      t = len(r.times)-1
      d = r.var(v, t, case)
      
      error += (100*L1(d, refdd))**2
      
      if 'wallclock_runtime_global' in r.runtime:
        rtime += r.runtime['wallclock_runtime_global']
      elif 'wallclock_runtime_total' in r.runtime:
        rtime += r.runtime['wallclock_runtime_total'] # legacy
      else:
        rtime += r.runtime['clock_runtime']
    
    errors.append ( numpy.sqrt(error/K) )
    ncells.append ( NX )
    rtimes.append ( rtime/K )
  
  # plot error vs ncells
  
  pylab.plot(numpy.log10(ncells), numpy.log10(errors), '-' + marker, label=name+label_suffix, color=color)
  pylab.title('Relative L1 error of ' + case + ' of ' + v.name + ' (K=%d)' % K)
  pylab.ylabel(r'log10($L^2(\Omega,\ L^1)$ - error')
  pylab.xlabel('log10(cells) in x-direction')
  pylab.legend(loc='lower left')
  pylab.grid(True)
  
  slopes = [[1,2], [1,1]]
  if not frame:
    for slope in slopes:
      plotSlope(slope, pylab.gca())
  
  if infolines:
    refr.showInfo('Reference ')
    pylab.subplots_adjust(bottom=0.25)
  
  if not frame:
    save = getFileName(refr, 'conv-vs-cells-' + case + save_suffix)
    pylab.savefig(save)
    refr.generateTexTable(save)
    pylab.show()

def plotErrorVsRuntime ( name, v, N, case='mean', frame=False, color=None, refr=None, K=1, infolines=False, save_suffix='', path='', label_suffix='', marker='x' ):
  '''Plots the errors for the refinements in the list N vs runtime'''
  
  reft = len(refr.times)-1
  refd = refr.var(v, reft, case)
  
  errors = []
  ncells = []
  rtimes = []
  
  for i , (NX, NY, NZ) in enumerate(N):
    
    refdd = refd
    
    zoom = refr.i.NX/NX
    if refr.i.NY == 1:
      while zoom > 1:
        refdd = (refdd[0:refr.i.NX-1:2,:,:]+refdd[1:refr.i.NX:2,:,:])/2
        zoom /= 2
    elif refr.i.NZ == 1:
      while zoom > 1:
        refdd = (refdd[0:refr.i.NX-1:2,0:refr.i.NY-1:2,:]+refdd[1:refr.i.NX:2,0:refr.i.NY-1:2,:]+refdd[1:refr.i.NX:2,1:refr.i.NY:2,:]+refdd[0:refr.i.NX-1:2,1:refr.i.NY:2,:])/4
        zoom /= 2
    else:
      while zoom > 1:
        refdd = (  refdd[0:refr.i.NX-1:2, 0:refr.i.NY-1:2, 0:refr.i.NZ-1:2] \
                 + refdd[0:refr.i.NX-1:2, 0:refr.i.NY-1:2, 1:refr.i.NZ:2  ] \
                 + refdd[0:refr.i.NX-1:2, 1:refr.i.NY:2,   0:refr.i.NZ-1:2] \
                 + refdd[0:refr.i.NX-1:2, 1:refr.i.NY:2,   1:refr.i.NZ:2  ] \
                 + refdd[1:refr.i.NX:2,   0:refr.i.NY-1:2, 0:refr.i.NZ-1:2] \
                 + refdd[1:refr.i.NX:2,   0:refr.i.NY-1:2, 1:refr.i.NZ:2  ] \
                 + refdd[1:refr.i.NX:2,   1:refr.i.NY:2,   0:refr.i.NZ-1:2] \
                 + refdd[1:refr.i.NX:2,   1:refr.i.NY:2,   1:refr.i.NZ:2  ] ) / 8
        zoom /= 2
    
    name_base = name + '_%dx%dx%d' % (NX, NY, NZ)
    
    error = 0
    rtime = 0
    for k in range(1, K+1):
      r = Run('%s-%d' % (name_base, k), path=path)
      
      t = len(r.times)-1
      d = r.var(v, t, case)
      
      error += (100*L1(d, refdd))**2
      
      if 'wallclock_runtime_global' in r.runtime:
        rtime += r.runtime['wallclock_runtime_global']
      elif 'wallclock_runtime_total' in r.runtime:
        rtime += r.runtime['wallclock_runtime_total'] # legacy
      else:
        rtime += r.runtime['clock_runtime']
    
    errors.append ( numpy.sqrt(error/K) )
    ncells.append ( NX )
    rtimes.append ( rtime/K )
  
  # plot error vs runtime
  
  pylab.plot(numpy.log10(rtimes), numpy.log10(errors), '-' + marker, label=name+label_suffix, color=color)
  pylab.title('Relative L1 error of ' + case + ' of ' + v.name + ' (K=%d)' % K)
  pylab.ylabel(r'log10($L^2(\Omega,\ L^1)$ - error')
  pylab.xlabel('log10(seconds)')
  pylab.legend(loc='lower left')
  pylab.grid(True)
  
  dim = int(refr.i.NX>1) + int(refr.i.NY>1) + int(refr.i.NZ>1)
  
  slopes = [[1,2+2*dim], [1,1+dim]]
  if not frame:
    for slope in slopes:
      plotSlope(slope, pylab.gca())
  
  if infolines:
    refr.showInfo('Reference ')
    pylab.subplots_adjust(bottom=0.25)
  
  if not frame:
    save = getFileName(refr, 'conv-vs-runtime-' + case + save_suffix)
    pylab.savefig(save)
    refr.generateTexTable(save)
    pylab.show()

def plotSlope(slope, plt, type='linear'):
  '''Plots the slope together with a numerical value in the plt axis'''
  
  from pylab import xlim, ylim
  from numpy import log10, power
  
  xmin, xmax = xlim()
  ymin, ymax = ylim()
  
  if type == 'linear':
    x1 = xmax - (xmax-xmin)/3
    x2 = xmax - (xmax-xmin)/10
    
    y1 = ymax - (ymax-ymin)/20
    y2 = y1 - float(slope[0])/slope[1] * (x2 - x1)
  
  else:
    x1 = log10(xmax) - (log10(xmax)-log10(xmin))/3
    x2 = log10(xmax) - (log10(xmax)-log10(xmin))/10
    
    y1 = log10(ymax) - (log10(ymax)-log10(ymin))/20
    y2 = y1 - float(slope[0])/slope[1] * (x2 - x1)
    
    x1 = power(10, x1)
    x2 = power(10, x2)
    y1 = power(10, y1)
    y2 = power(10, y2)
  
  plt.plot([x1, x2], [y1, y2], '--k', label='_nolegend_')
  plt.annotate(' %d/%d' % (slope[0], slope[1]), xy=(x2, y2),  xycoords='data')

def plotEfficiency ( name, N, frame=False, color=None, K=1, save_suffix='', path='', label_suffix='', marker='x' ):
  '''Plots the parallelisation efficiency for the refinements in the list N'''
  
  ncells = []
  
  wallclock_runtimes_global     = []
  wallclock_runtimes_global_mpi = []
  mpi_efficiencies              = []
  
  for i, (NX, NY, NZ) in enumerate(N):
    
    name_base = name + '_%dx%dx%d' % (NX, NY, NZ)
    
    wallclock_runtime_global     = 0
    wallclock_runtime_global_mpi = 0
    mpi_efficiency               = 0
    
    for k in range(1, K+1):
      container = {}
      execfile(path + '%s_%s-%d.py' % ('runtime', name_base, k), globals(), container)
      
      if 'wallclock_runtime_global' in container:
        wallclock_runtime_global += container['wallclock_runtime_global']
      else:
        wallclock_runtime_global += container['wall_runtime_total'] # legacy
      
      if 'wallclock_runtime_global_mpi' in container:
        wallclock_runtime_global_mpi += container['wallclock_runtime_global_mpi']
      else:
        wallclock_runtime_global_mpi += container['mpi_wall_runtime_total'] # legacy
      
      if 'mpi_efficiency' in container:
        mpi_efficiency += container['mpi_efficiency']
      else:
        mpi_efficiency += container['mpi_scaling'] # legacy
    
    ncells.append ( NX * NY * NZ )
    
    wallclock_runtimes_global     .append ( wallclock_runtime_global/K )
    wallclock_runtimes_global_mpi .append ( wallclock_runtimes_global_mpi/K )
    mpi_efficiencies              .append ( mpi_efficiencies/K )
  
  # runtimes vs ncells
  
  pylab.subplot(121)
  pylab.plot(numpy.log10(ncells), numpy.log10(wallclock_runtimes_global), '-' + marker, label=name+label_suffix, color=color)
  pylab.plot(numpy.log10(ncells), numpy.log10(wallclock_runtimes_global_mpi), color+'--')
  pylab.title('Wall runtime vs. MPI calls | avg. of %d runs' % K)
  pylab.ylabel('log10(seconds)')
  pylab.xlabel('log10(cells)')
  pylab.legend(loc='best')
  pylab.grid(True)
  
  # MPI efficiency vs ncells
  
  pylab.subplot(122)
  pylab.plot(numpy.log10(ncells), mpi_efficiencies, '-' + marker, label=name+label_suffix, color=color)
  pylab.ylim(0,1.02)
  pylab.title('MPI efficiency | average of %d runs' % K)
  pylab.ylabel('efficiency')
  pylab.xlabel('log10(cells)')
  pylab.legend(loc='lower right')
  pylab.grid(True)
  
  pylab.subplots_adjust(right=0.98)
  pylab.subplots_adjust(left=0.06)
  pylab.subplots_adjust(top=0.92)
  pylab.subplots_adjust(bottom=0.12)
  
  if not frame:
    pylab.savefig(getFileName(Run(name_base + '-1'), 'efficiency' + save_suffix))
    pylab.show()

def plotStrongScaling ( name, N, frame=False, color=None, K=1, save_suffix='', ref_start=1, path='', label_suffix='', marker='x' ):
  '''Plots the strong scaling for the refinements in the list N'''
  
  # === strong scaling
  
  wallclock_runtimes = []
  mpi_efficiencies   = []  
  cores              = []
  
  for i in range(ref_start, len(N)+1):
    
    name_base = name + '_%d' % i
    
    wallclock_runtime = 0
    mpi_efficiency    = 0
    
    for k in range(1, K+1):
      container = {}
      execfile(path + '%s_%s-%d.py' % ('runtime', name_base, k), globals(), container)
      
      if 'wallclock_runtime' in container:
        wallclock_runtime += container['wallclock_runtime']
      else:
        wallclock_runtime += container['wall_runtime'] # legacy
      
      if 'mpi_efficiency' in container:
        mpi_efficiency += container['mpi_efficiency']
      else:
        mpi_efficiency += container['mpi_scaling'] # legacy
    
    if 'cores' in container:
      cores .append( container['cores'] )
    else:
      cores .append( container['nodes'] ) # legacy
    
    wallclock_runtimes .append ( wallclock_runtime/K )
    mpi_efficiencies   .append ( mpi_efficiency/K )
  
  pylab.subplot(121)
  pylab.loglog(cores, wallclock_runtimes, '-' + marker, label=name+label_suffix, color=color)
  if K != 1:
    pylab.title('Strong scaling | average of %d runs' % K)
  else:
    pylab.title('Strong scaling')
  pylab.ylabel('wallclock runtime')
  pylab.xlabel('cores')
  pylab.legend(loc='lower left')
  pylab.grid(True)
  
  if not frame:
    plotSlope([1,1], pylab.gca(), type='loglog')
  
  # === efficiency
  
  pylab.subplot(122)
  pylab.semilogx(cores, mpi_efficiencies, '-' + marker, label=name+label_suffix, color=color)
  pylab.ylim(0,1.02)
  if K != 1:
    pylab.title('MPI efficiency | average of %d runs' % K)
  else:
    pylab.title('MPI efficiency')
  pylab.ylabel('efficiency')
  pylab.xlabel('cores')
  pylab.legend(loc='lower center')
  pylab.grid(True)
  
  # === formatting
  
  pylab.subplots_adjust(right=0.97)
  pylab.subplots_adjust(left=0.07)
  pylab.subplots_adjust(top=0.93)
  pylab.subplots_adjust(bottom=0.15)
  
  if not frame:
    pylab.savefig(getFileName(Run(name_base + '-1', load_times=False), 'scaling-strong' + save_suffix))
    pylab.show()

def plotWeakScaling ( name, N, frame=False, color=None, K=1, save_suffix='', path='', label_suffix='', marker='x' ):
  '''Plots the weak scaling for the refinements in the list N'''
  
  # === weak scaling
  
  wallclock_runtimes_global = []
  mpi_efficiencies          = []
  cores                     = []
  
  for i, (NX, NY, NZ) in enumerate(N):
    
    name_base = name + '_%dx%dx%d' % (NX, NY, NZ)
    
    wallclock_runtime_global = 0
    mpi_efficiency           = 0
    
    for k in range(1, K+1):
      container = {}
      execfile(path + '%s_%s-%d.py' % ('runtime', name_base, k), globals(), container)
      
      if 'wallclock_runtime_global' in container:
        wallclock_runtime_global += container['wallclock_runtime_global']
      else:
        wallclock_runtime_global += container['wall_runtime_total'] # legacy
      
      if 'mpi_efficiency' in container:
        mpi_efficiency += container['mpi_efficiency']
      else:
        mpi_efficiency += container['mpi_scaling'] # legacy
    
    if 'cores' in container:
      cores .append( container['cores'] )
    else:
      cores .append( container['nodes'] ) # legacy
    
    r = Run( '%s-%d' % (name_base, 1), load_times=False )
    dim = int(r.i.NX>1) + int(r.i.NY>1) + int(r.i.NZ>1)
    order = int(r.vars['ORDER'])
    
    log_factor = 1
    if dim+1 == order:
      log_factor = ( i + numpy.log2(N[0][0]) ) / numpy.log2(N[0][0])
    
    wallclock_runtimes_global .append ( (wallclock_runtime_global/K) / ((2**(float(dim+1)/dim))**(2*i) * log_factor) )
    mpi_efficiencies          .append ( mpi_efficiency/K )
  
  pylab.subplot(121)
  pylab.loglog(cores, wallclock_runtimes_global, '-' + marker, label=name+label_suffix, color=color)
  if K != 1:
    pylab.title('Weak scaling | average of %d runs' % K)
  else:
    pylab.title('Weak scaling')
  pylab.ylabel('global wallclock-time')
  pylab.xlabel('cores (~ problem size)')
  pylab.legend(loc='best')
  pylab.grid(True)
  
  # === efficiency
  
  pylab.subplot(122)
  pylab.semilogx(cores, mpi_efficiencies, '-' + marker, label=name+label_suffix, color=color)
  pylab.ylim(0,1.02)
  if K != 1:
    pylab.title('MPI efficiency | average of %d runs' % K)
  else:
    pylab.title('MPI efficiency')
  pylab.ylabel('efficiency')
  pylab.xlabel('cores (~ problem size)')
  pylab.legend(loc='lower center')
  pylab.grid(True)
  
  # === formatting
  
  pylab.subplots_adjust(right=0.98)
  pylab.subplots_adjust(left=0.06)
  pylab.subplots_adjust(top=0.93)
  pylab.subplots_adjust(bottom=0.15)
  
  if not frame:
    pylab.savefig(getFileName(Run(name_base + '-1', load_times=False), 'scaling-weak' + save_suffix))
    pylab.show()

def plotStochasticScaling ( name, KL, frame=False, color=None, K=1, save_suffix='', path='', label_suffix='', marker='x' ):
  '''Plots the stochastic scaling for the number of stochastic dimensions in the list KL'''
  
  rtimes = []
  
  for m in KL:
    
    if name == '':
      name_base = 'KL=%d' % m
    else:
      name_base = name + '_KL=%d' % m
    
    rtime = 0
    
    for k in range(1, K+1):
      container = {}
      execfile(path + '%s_%s-%d.py' % ('runtime', name_base, k), globals(), container)
      if 'wallclock_runtime_global' in r.runtime:
        rtime += r.runtime['wallclock_runtime_global']
      elif 'wallclock_runtime_total' in r.runtime:
        rtime += r.runtime['wallclock_runtime_total'] # legacy
      else:
        rtime += r.runtime['clock_runtime']
      
    rtimes.append ( rtime/K )
  
  pylab.plot(numpy.log10(KL), numpy.log10(rtimes), '-' + marker, label=name+label_suffix, color='k')
  pylab.title('Stochastic scaling | average of %d runs' % K)
  pylab.ylabel('log10(seconds)')
  pylab.xlabel('log10(KL-coefficients)')
  pylab.legend(loc='upper left')
  pylab.grid(True)
  
  if not frame:
    plotSlope([-1,1], pylab.gca())
  
  Run('%s-1' % name_base).showInfo()
  
  # === formatting
  
  pylab.subplots_adjust(right=0.96)
  pylab.subplots_adjust(left=0.10)
  pylab.subplots_adjust(top=0.94)
  
  if not frame:
    pylab.savefig(getFileName(Run(name_base + '-1', load_times=False), 'scaling-stochastic' + save_suffix))
    pylab.show()

# exporters

def save_matlab ( name, v, N, case='mean', K=1, save_suffix='', matlab_path='', path='' ):
  '''Saves all refinement data to Matlab files'''
  
  for i , (NX, NY, NZ) in enumerate(N):
    
    name_base = name + '_%dx%dx%d' % (NX, NY, NZ)
    
    for k in range(1, K+1):
      r = Run('%s-%d' % (name_base, k), path=path)
      r.matlab(br.U, path=matlab_path)

# routines creating tables

def tableref(r, runref, v):
    fmt="%10.1e"
    scale=1
    t1 = len(r.times)-1
    t2 = len(runref.times)-1
    d = r.var(v, t1)
    refd = runref.var(v, t2)
    zoom = runref.i.NX/r.i.NX
    if runref.i.NY == 1:
        while zoom>1:
            refd = (refd[0:runref.i.NX-1:2,:,:]+refd[1:runref.i.NX:2,:,:])/2
            zoom=zoom/2
    else:
        while zoom>1:
            refd = (refd[0:runref.i.NX-1:2,0:runref.i.NY-1:2,:]+refd[1:runref.i.NX:2,0:runref.i.NY-1:2,:]+refd[1:runref.i.NX:2,1:runref.i.NY:2,:]+refd[0:runref.i.NX-1:2,1:runref.i.NY:2,:])/4
            zoom=zoom/2
    td = (L1(d, refd)*scale, Linf(d, refd)*scale)
    print ((r"%s & %s & %s \\"% (r.name,fmt,fmt)) % td)

def rebin(d, f):
    nx,ny,nz = d.shape
    if ny == 1:
        assert nz == 1
        return sum(d[i:nx-f+1+i:f,:,:] for i in range(f))/f
    elif nz == 1:
        return sum(d[i:nx-f+1+i:f,i:ny-f+1+i:f,:] for i in range(f))/(f*f)
    else:
        return sum(d[i:nx-f+1+i:f,i:ny-f+1+i:f,i:nz-f+1+i:f] for i in range(f))/(f*f*f)

def table(runs, v, t=None, fmt="%12.1e", refd=None, reft=None, refr=None, scale=1, dt=0.01):
    if t is None:
        t = max(r.times[-1] for r in runs)
    print (r'       Run &        $L_1$ &   $L_\infty$ \\')
    for r in runs:
        ts = None
        for n,tm in enumerate(r.times):
            if abs(tm - t) < dt:
                ts = n
                break
        if ts is None:
            print (r.name + r' & crashed \\')
            continue
        d = r.var(v, ts)
        if reft is not None:
            refts = None
            for n,tm in enumerate(r.times):
                if tm == reft:
                    refts = n
                    break
            refd = r.var(v, refts)
        elif refr is not None:
            refd = refr.var(v, ts)
            if refr.i.NX != r.i.NX:
                assert (refr.i.NX//r.i.NX) * r.i.NX == refr.i.NX
                refd = rebin(refd, refr.i.NX // r.i.NX)
        td = (L1(d, refd)*scale, Linf(d, refd)*scale)
        print ((r"%10s & %s & %s \\"% (r.name,fmt,fmt)) % td)

# helpers

make1d_N = lambda refs, start=32 : [[start*2**i,1,1] for i in range(refs)]
make2d_N = lambda refs, start=32 : [[start*2**i,start*2**i,1] for i in range(refs)]
make3d_N = lambda refs, start=32 : [[start*2**i,start*2**i,start*2**i] for i in range(refs)]

# needed for weak scaling
def make2d_N_alternating (refs, start=32):
  N = []
  tmp  = 0
  i_nx = 0
  i_ny = 0
  
  for ref in range(refs):
    
    N.append([start*2**i_nx,start*2**i_ny,1])
    
    if tmp == 0:
      i_nx += 1
      tmp = 1
    else:
      i_ny += 1
      tmp = 0
  
  return N

# loads the reference solution from the required directory
def load_reference (ref_path=None):
  if ref_path == None:
    r = Run()
    print ( 'Reference solution loaded from the current directory.')
  else:
    r = Run(path=ref_path)
    print ( 'Reference solution loaded from: ' + ref_path)
  
  return r
