# -*- coding: utf-8 -*-

# routines for generation of 'bc.cpp' file (boundary conditions)

BC_HEADER = "\n// This file is automatically generated by 'make.py' script\n\n"
BC_HEADER_SINGLE = BC_HEADER + '#include "equation.h"\n'
BC_HEADER_PARALLEL = BC_HEADER_SINGLE
BC_HEADER_PARALLEL += '#include "multi_mpi.h"\n'
BC_HEADER_PARALLEL += '#include "mpi.h"\n'

BC_FUNC_VALUES = ['N', 'l', 'N', 'l', 'N', 'l']
BC_STD_VALUES  = ['i', 'j', 'k', 'i', 'j', 'k']

BC_STD = "U[%s][%s][%s] = U[%s][%s][%s]"
BC_FIX = "bc_fix(%s,%s,%s,%s,%s,%s)"

BC_CALL_DEF  = "void bc_%s_%s%s(ConservedVars &U, int i, int j, int k);\n"
BC_CALL      = "bc_%s_%s%s(U[%s][%s][%s],i,j,k)"
BC_INIT_CALL = "bc_%s_%s%s_init();\n"

BC_FUNC_SINGLE = """
void set_ghost_%s()
{
  for (int i = 0; i < %sX%s; i++) {
    for (int j = 0; j < %sY%s; j++) {
      for (int k = 0; k < %sZ%s; k++) {
        %s;
        %s;
        calc_xtra(Xtra[%s][%s][%s], U[%s][%s][%s], %s, %s, %s);
        %s;
        %s;
        calc_xtra(Xtra[%s][%s][%s], U[%s][%s][%s], %s, %s, %s);
      }
    }
  }
}
"""

BC_FUNC_PARALLEL = """
void set_ghost_%s()
{
  for (int i = 0; i < %sX%s; i++) {
    for (int j = 0; j < %sY%s; j++) {
      for (int k = 0; k < %sZ%s; k++) {
        if (mpibc[%s]) {
          U[%s][%s][%s] = mpi_rbuf[mpi_bc_rbuf[%s]][(i*%sY%s + j)*%sZ%s + k];
        } else {
          %s;
          %s;
        }
        calc_xtra(Xtra[%s][%s][%s], U[%s][%s][%s], %s, %s, %s);
        if (mpibc[%s]) {
          U[%s][%s][%s] = mpi_rbuf[mpi_bc_rbuf[%s]][(i*%sY%s + j)*%sZ%s + k];
        } else {
          %s;
          %s;
        }
        calc_xtra(Xtra[%s][%s][%s], U[%s][%s][%s], %s, %s, %s);
      }
    }
  }
}
"""

def bc_single (f, spec):
    f.write(BC_HEADER_SINGLE)
    init = []
    for n in range(6):
        if spec[n] == 'REPEAT': spec[n] = 1
        elif spec[n] == 'NEUMANN': spec[n] = 2
        else:
            if spec[n][-1] == '*':
                spec[n] = spec[n][:-1]
                d = (spec[n].lower(), "xyzxyz"[n], "llluuu"[n])
                init.append('  ' + BC_INIT_CALL % d)
                f.write('void ' + BC_INIT_CALL % d)
            else:
                d = (spec[n].lower(), "xyzxyz"[n], "llluuu"[n])
            f.write(BC_CALL_DEF % d)
    for b in range(3):
        fv = ['xyz'[b]] + BC_FUNC_VALUES
        fv[1+2*b]   = 'G'
        fv[1+2*b+1] = ''
        lv = BC_STD_VALUES + []
        lv[b] = '-1-' + lv[b]
        uv = BC_STD_VALUES + []
        uv[b] += '+N' + 'XYZ'[b] + 'l'
        lsrc = lv[:3]*3
        usrc = uv[:3]*3
        if spec[b] == 1:
            lv[3+b] = 'N' + 'XYZ'[b] + 'l' + '-1-' + lv[3+b]
        elif spec[b] != 2:
            ls = BC_CALL
            lv = [spec[b].lower(), 'xyz'[b], 'l'] + lv[:3]
        if spec[b+3] == 2:
            uv[3+b] = 'N' + 'XYZ'[b] + 'l' + '-1-' + uv[3+b]
        elif spec[b+3] != 1:
            us = BC_CALL
            uv = [spec[b+3].lower(), 'xyz'[b], 'u']+uv[:3]
        fv += [BC_STD % tuple(lv)] + [BC_FIX % tuple(lv)] + lsrc + [BC_STD % tuple(uv)] + [BC_FIX % tuple(uv)] + usrc
        f.write(BC_FUNC_SINGLE % tuple(fv))
    f.write("\nvoid bc_init()\n{\n"+''.join(init)+"}\n")
    f.close()

def bc_parallel (f, spec, settings):
    f.write(BC_HEADER_PARALLEL)
    init = []
    for n in range(6):
        if spec[n] == 'REPEAT':
            spec[n] = 1
            settings[1]['BC_REPEAT_' + 'XYZ'[n%3]] = 1
        elif spec[n] == 'NEUMANN': spec[n] = 2
        else:
            if spec[n][-1] == '*':
                spec[n] = spec[n][:-1]
                d = (spec[n].lower(), "xyzxyz"[n], "llluuu"[n])
                init.append('  ' + BC_INIT_CALL % d)
                f.write('void ' + BC_INIT_CALL % d)
            else:
                d = (spec[n].lower(), "xyzxyz"[n], "llluuu"[n])
            f.write(BC_CALL_DEF % d)
    for b in range(3):
        fv = ['xyz'[b]] + BC_FUNC_VALUES
        fv[1+2*b]   = 'G'
        fv[1+2*b+1] = ''
        lm = ['XYZ'[b]] + ['i','j','k'] + ['XYZ'[b]] + ['N', 'l', 'N', 'l']
        lm[1+b] = '-1-' + lm[1+b]
        if b == 1:
          lm[5] = 'G'
          lm[6] = ''
        if b == 2:
          lm[7] = 'G'
          lm[8] = ''
        um = ['XYZ'[b] + '+NDIR'] + ['i','j','k'] + ['XYZ'[b] + '+NDIR'] + ['N', 'l', 'N', 'l']
        um[1+b] = 'N' + 'XYZ'[b] + 'l' + '+' + um[1+b]
        if b == 1:
          um[5] = 'G'
          um[6] = ''
        if b == 2:
          um[7] = 'G'
          um[8] = ''
        ls = BC_STD
        lv = BC_STD_VALUES + []
        us = BC_STD
        uv = BC_STD_VALUES + []
        lv[b] = '-1-' + lv[b]
        uv[b] += '+N' + 'XYZ'[b] + 'l'
        lsrc = lv[:3]*3
        usrc = uv[:3]*3
        if spec[b] == 1:
            lv[3+b] = 'N' + 'XYZ'[b] + 'l' + '-1-' + lv[3+b]
        elif spec[b] != 2:
            ls = BC_CALL
            lv = [spec[b].lower(), 'xyz'[b], 'l'] + lv[:3]
        if spec[b+3] == 2:
            uv[3+b] = 'N' + 'XYZ'[b] + 'l' + '-1-' + uv[3+b]
        elif spec[b+3] != 1:
            us = BC_CALL
            uv = [spec[b+3].lower(), 'xyz'[b], 'u']+uv[:3]
        fv += lm + [BC_STD % tuple(lv)] + [BC_FIX % tuple(lv)] + lsrc + um + [BC_STD % tuple(uv)] + [BC_FIX % tuple(uv)] + usrc
        f.write(BC_FUNC_PARALLEL % tuple(fv))
    f.write("\nvoid bc_init()\n{\n"+''.join(init)+"}\n")
    f.close()
