
import pylab
import os

from variables import *

log10_for_variance_treshhold = 100
log10_for_variance_cutoff    = 1e-2
colors = ['b', 'g', 'r', 'c', 'm', 'y', 'burlywood', 'chartreuse', 'k']

line_format_single    = 'b'
line_format_upper_std = 'g--'
line_format_lower_std = 'r--'

# for B&W plots only
#line_format_single    = 'k'
#line_format_upper_std = 'k:'
#line_format_lower_std = 'k--'

import matplotlib as mpl
mpl.rcParams['font.size'] = 18
mpl.rcParams['legend.fontsize'] = 16

class Run (object):
    def __init__(self, name=None, path='', load_times=True):
        if name:
            self.outputf = path + 'output_'+name+'.dat'
            self.varf = path + 'var_output_'+name+'.dat'
            self.debugf = path + 'debug_output_'+name+'.dat'
            self.timesf = path + 'times_'+name+'.py'
            self.configf = path + 'config_'+name+'.py'
            self.runtimef = path + 'runtime_'+name+'.py'
            self.histf = path + 'hist_data_'+name+'.py'
            self.movief = path + 'movie_'+name
            self.name = name
        else:
            self.outputf = path + 'output.dat'
            self.varf = path + 'var_output.dat'
            self.debugf = path + 'debug_output.dat'
            self.timesf = path + 'times.py'
            self.configf = path + 'config.py'
            self.runtimef = path + 'runtime.py'
            self.histf = path + 'hist_data.py'
            self.movief = path + 'movie'
            self.name = path + 'run'
        self.load_times = load_times
        if load_times:
            times = {}
            execfile(self.timesf, globals(), times)
            self.times = times['time']
        config = {}
        execfile(self.configf, globals(), config)
        self.opts = config['opts']
        self.vars = config['used']
        if 'info' in config:
          self.info = config['info']
        else:
          self.info = None
        self.runtime = {}
        execfile(self.runtimef, globals(), self.runtime)
        self.i = Config(self, int)
        self.f = Config(self, float)
        self.nx = self.i.NX
        self.ny = self.i.NY
        self.nz = self.i.NZ
        if 'HISTOGRAMS' in self.vars and self.vars['HISTOGRAMS'] == '1':
          self.hist = {}
          execfile(self.histf, globals(), self.hist)
        else:
          self.hist = None
        
        # extracting the data (infolines) to be displayed on the plots
        self.info_opts = self.opts
        self.info_vars = config['vars']
        if 'RUNTIME_FILE'   in self.info_vars: del self.info_vars ['RUNTIME_FILE']
        if 'LOG_FILE'       in self.info_vars: del self.info_vars ['LOG_FILE']
        if 'TIME_FILE'      in self.info_vars: del self.info_vars ['TIME_FILE']
        if 'HIST_DATA_FILE' in self.info_vars: del self.info_vars ['HIST_DATA_FILE']
        if 'DATA_FILE'      in self.info_vars: del self.info_vars ['DATA_FILE']
        if 'DATA_PATH'      in self.info_vars: del self.info_vars ['DATA_PATH']
        
        # detecting number of variables from equation type
        if 'equation' in self.opts:
          self.NVARS = get_NVARS (self.opts['equation'])
        else:
          self.NVARS = 16
    
    def var(self, v, ts, case="mean"):
        if self.load_times and ts > len(self.times):
            raise Exception ("Timestep out of bounds")
        if case == "mean":
            f = open(self.outputf, 'rb')
        elif case == "variance":
            f = open(self.varf, 'rb')
        elif case == "debug":
            f = open(self.debugf, 'rb')
        else:
            raise Exeption ("Data file for '" + case + "' does not exist")
        n = self.nx*self.ny*self.nz
        f.seek(8*n*(v.i + self.NVARS*ts))
        fd = numpy.fromfile(f, dtype=numpy.float64, count=n)
        f.close()
        fd.shape = (self.nx, self.ny, self.nz)
        return fd
    
    def showInfo(self, prefix='', adjust=True):
      
      # filter out default alternative choices to reduce output size
      info_opts = self.info_opts
      if 'material' in info_opts and info_opts['material'] == 'none':
        del info_opts['material']
      if 'topography' in info_opts and info_opts['topography'] == 'none':
        del info_opts['topography']
      
      # create strings from dictionaries
      string_opts = ''.join([' %s:%s,' % x for x in sorted(info_opts.items())])
      string_vars = ''.join([' %s=%s' % x for x in sorted(self.info_vars.items())])
      
      text  = prefix + 'OPTS:' + string_opts[0:-1] + '\n'
      text += prefix + 'VARS:' + string_vars + '\n'
      
      if 'wallclock_runtime' in self.runtime:
        runtime = self.runtime['wallclock_runtime']
      elif 'wall_runtime' in self.runtime:
        runtime = self.runtime['wall_runtime'] # legacy
      else:
        runtime = self.runtime['clock_runtime']
      
      import datetime
      runtime = str(datetime.timedelta(seconds=round(runtime)))
      
      text += prefix + 'INFO: '
      if self.info:
        text += 'cluster: ' + str(self.info['CLUSTER']) + ', '
      
      if 'cores' in self.runtime:
        text += 'cores: ' + str(self.runtime['cores']) + ', ' + 'runtime: ' + runtime
      else: # legacy
        text += 'cores: ' + str(self.runtime['nodes']) + ', ' + 'runtime: ' + runtime
      
      if 'mpi_efficiency' in self.runtime:
        text += ', ' + 'efficiency: ' + str(round(1000 * self.runtime['mpi_efficiency']) / 10.0) + r'%'
      
      # legacy
      if 'mpi_scaling' in self.runtime:
        text += ', ' + 'efficiency: ' + str(round(1000 * self.runtime['mpi_scaling']) / 10.0) + r'%'
      
      if 'wallclock_runtime_global_mpi_ddm' in self.runtime:
        runtime_global = self.runtime['wallclock_runtime_global']
        text += ', ' + 'overhead: '
        text += ' DDM ' + str(round(100 * self.runtime['wallclock_runtime_global_mpi_ddm']/runtime_global)) + r'%'
        text += ', MC ' + str(round(100 * self.runtime['wallclock_runtime_global_mpi_mc']/runtime_global)) + r'%'
        text += ', MLMC ' + str(round(100 * self.runtime['wallclock_runtime_global_mpi_mlmc']/runtime_global)) + r'%'
        text += ', idle ' + str(round(100 * self.runtime['wallclock_runtime_global_mpi_idle']/runtime_global)) + r'%'
      
      pylab.figtext(0.01, 0.015, text, fontsize='small')
      
      if adjust:
        pylab.subplots_adjust(bottom=0.20)
    
    def generateTexTable(self, save=None):
      
      # config
      
      keys     =  ['L',    'ML',     'grid_size', 'CFL', 'cores', 'runtime']
      captions =  [r'$L$', r'$M_L$', 'grid size', 'CFL', 'cores', 'runtime']
      
      if 'mpi_efficiency' or 'mpi_scaling' in self.runtime:
        keys     += ['efficiency']
        captions += ['efficiency']
      
      # aggregation of information
      
      opts = {}
      opts.update(self.opts)
      opts.update(self.vars)
      
      grid_size = str(opts['NX'])
      if self.ny != 1:
        grid_size += 'x' + str(opts['NY'])
      if self.nz != 1:
        grid_size += 'x' + str(opts['NZ'])
      opts.update( { 'grid_size' : grid_size } )
      
      if 'cores' in self.runtime:
        opts.update( { 'cores' : str(self.runtime['cores']) } )
      else: # legacy
        opts.update( { 'cores' : str(self.runtime['nodes']) } )
      
      if 'wallclock_runtime' in self.runtime:
        runtime = self.runtime['wallclock_runtime']
      elif 'wall_runtime' in self.runtime:
        runtime = self.runtime['wall_runtime'] # legacy
      else:
        runtime = self.runtime['clock_runtime']
      
      import datetime
      runtime = str(datetime.timedelta(seconds=round(runtime)))
      
      opts.update( { 'runtime' : str(runtime) } )
      
      if 'mpi_efficiency' in self.runtime:
        opts.update( { 'efficiency' : str(round(1000 * self.runtime['mpi_efficiency']) / 10.0) + r'\%' } )
      
      # legacy
      if 'mpi_scaling' in self.runtime:
        opts.update( { 'efficiency' : str(round(1000 * self.runtime['mpi_scaling']) / 10.0) + r'\%' } )
      
      # TeX code generation
      
      columns =  '|' + 'c|' * len(keys)
      
      text =  '\n'
      text += r'\begin{tabular}{%s}' % columns + '\n'
      
      text += r'\hline' + '\n'
      text += (r'%s & ' * len(captions))[0:-2] % tuple(captions) + r'\\' + '\n'
      text += r'\hline' + '\n'
      text += (r'%s & ' * len(keys))[0:-2] % tuple([opts[key] for key in keys]) + r'\\' + '\n'
      text += r'\hline' + '\n'
      
      text += r'\end{tabular}' + '\n'
      
      # saving
      
      f = open(save[0:-4] + '.tex', 'w')
      f.write(text)
      f.close()
    
    def plotTopography_1d(self, xr, ts):
        b_mean = self.var(sw.B, ts, "mean")[:,0,0]
        
        if int(self.vars['COMPUTE_VARIANCE']) == 0:
          pylab.plot(xr, b_mean, "k-", label="topography")
        else:
          pylab.plot(xr, b_mean, "k-")
        
        if int(self.vars['COMPUTE_VARIANCE']) == 1:
          b_var = self.var(sw.B, ts, "variance")[:,0,0]
          b_std = numpy.sqrt(b_var)
          pylab.plot(xr, b_mean + b_std, "k--")
        
        ymin, ymax = pylab.ylim()
        pylab.fill_between(xr, b_mean, ymin, alpha=0.5, color='black')
    
    def plotTopography_2d(self, extent, ts, case, N=4):
        b = self.var(sw.B, ts, case)[:,:,0]
        cp = pylab.contour(numpy.transpose(b), N, extent=extent, colors='black')
    
    def plot(self, v, frame=False, ts=None, extent=None, infolines=True, save=None, topography=True, loc='best', cut_slice=None, contour=False, log_var=False, poster=False):
        if ts is None: ts = len(self.times)-1
        if self.ny == 1:
            self.plot_1d(v, ts, extent, frame, infolines, save, topography, loc)
        else:
            self.subplot_2d(v, ts, extent, extent, frame, infolines, save, topography=False, cut_slice=cut_slice, contour=contour, log_var=log_var, poster=poster)
    
    def plot0(self, v, frame=False, ts=None, extent=None, infolines=True, save=None, topography=True, loc='best', cut_slice=None, contour=False, log_var=False):
        if not int(self.vars['SAVE0']):
            raise Exception ("Snapshot at t=0 is not available! Use SAVE0=1 when compiling.")
        
        if ts is None: ts = len(self.times)-1
        
        if self.ny == 1:
            if not frame:
              if infolines:
                pylab.figure(figsize=(2*8,6))
              else:
                pylab.figure(figsize=(2*8,5))
            pylab.subplot(121)
            self.plot_1d(v, 0, extent, True, False, None, topography=topography, loc=loc)
            pylab.subplot(122)
            self.plot_1d(v, ts, extent, frame, infolines, save, topography=topography, loc=loc)
        else:
            self.subplot_2d(v, 0, extent,  extent, frame, infolines, save, topography, cut_slice, contour=contour, log_var=log_var)
            self.subplot_2d(v, ts, extent, extent, frame, infolines, save, topography, cut_slice, contour=contour, log_var=log_var)
    
    def subplot(self, v, frame=False, ts=None, extent_mean=None, extent_var=None, infolines=True, save=None, topography=True, movie=False, loc='best', cut_slice=None, contour=False, log_var=False):
        if ts is None: ts = len(self.times)-1
        if self.ny == 1:
            self.subplot_1d(v, ts, extent_mean, extent_var, frame, infolines, save, movie, loc=loc, topography=topography)
        else:
            self.subplot_2d(v, ts, extent_mean, extent_var, frame, infolines, save, topography=False, cut_slice=cut_slice, movie=movie, contour=contour, log_var=log_var)

    def plot_1d(self, v, ts, extent=None, frame=False, infolines=True, save=None, x=None, y=None, z=None, topography=True, loc='best'):
        xr = numpy.arange(self.nx)*(self.f.MAXX/self.nx)+0.5*self.f.MAXX/self.nx
        mean = self.var(v, ts, "mean")
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H and (not 'stats' in self.opts or self.opts['stats'] == 'mean-var'):
          mean += self.var(sw.B, ts, "mean")
        
        if int(self.vars['L']) != 0 or int(self.vars['ML']) != 1:
          mean_label = "mean"
        elif self.name[-3:] != "run":
          mean_label = self.name
        else:
          mean_label = "solution"
        
        pylab.plot(xr, mean[:,0,0], line_format_single, label=mean_label)
        
        if not 'COMPUTE_VARIANCE' in self.vars or int(self.vars['COMPUTE_VARIANCE']) == 1:
          var = self.var(v, ts, "variance")
          if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H and (not 'stats' in self.opts or self.opts['stats'] == 'mean-var'):
            var -= self.var(sw.B, ts, "variance")
          std = numpy.sqrt(numpy.fabs(var));
          upper_std = mean + std;
          lower_std = mean - std;
          pylab.plot(xr, upper_std[:,0,0], line_format_upper_std, label="mean + std")
          pylab.plot(xr, lower_std[:,0,0], line_format_lower_std, label="mean - std")
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and topography and v == sw.H:
          self.plotTopography_1d(xr, ts)
        
        if extent:
            pylab.ylim(*extent)
        pylab.title(v.name + ' at t=%.2f' % self.times[ts])
        
        pylab.legend(loc=loc)
        
        pylab.subplots_adjust(right=0.96)
        pylab.subplots_adjust(left=0.06)
        # bugfix in pylab for topography plotting
        if not 'equation' in self.opts or not self.opts['equation'] == 'sw' or not topography or not v == sw.H:
          pylab.subplots_adjust(top=0.94)
        
        if infolines: self.showInfo()
        
        if save:
            pylab.savefig(save)
            self.generateTexTable(save)
        if not frame:
            pylab.show()
    
    def subplot_1d(self, v, ts, extent_mean=None, extent_var=None, frame=False, infolines=True, save=None, movie=False, x=None, y=None, z=None, topography=True, loc='best'):
        
        if movie:
          pylab.figure(figsize=(2*8,6))
        
        xr = numpy.arange(self.nx)*(self.f.MAXX/self.nx)+0.5*self.f.MAXX/self.nx
        
        d = self.var(v, ts, 'mean')
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H and (not 'stats' in self.opts or self.opts['stats'] == 'mean-var'):
          d += self.var(sw.B, ts, "mean")
        
        if int(self.vars['L']) != 0 or int(self.vars['ML']) != 1:
          title_prefix = "mean of "
        else:
          title_prefix = ""
        
        pylab.subplot(121)
        pylab.plot(xr, d[:,0,0], line_format_single, label=self.name)
        pylab.title(title_prefix + v.name + ' at t=%.2f' % self.times[ts])
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and topography and v == sw.H:
          self.plotTopography_1d(xr, ts)
        
        pylab.legend(loc=loc)
        
        if extent_mean:
            pylab.ylim(*extent_mean)
        
        if not 'COMPUTE_VARIANCE' in self.vars or int(self.vars['COMPUTE_VARIANCE']) == 1:
          
          d = self.var(v, ts, 'variance')
          if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H and (not 'stats' in self.opts or self.opts['stats'] == 'mean-var'):
            d = numpy.fabs(d - self.var(sw.B, ts, "variance"))
          
          pylab.subplot(122)
          pylab.plot(xr, d[:,0,0], line_format_single, label=self.name)
          if extent_var:
              pylab.ylim(*extent_var)
          if not 'COMPUTE_KURTOSIS' in self.vars or int(self.vars['COMPUTE_KURTOSIS']) == 0:
            pylab.title('variance' + ' of ' + v.name + ' at t=%.2f' % self.times[ts])
          else:
            pylab.title('4th centered moment' + ' of ' + v.name + ' t=%.2f' % self.times[ts])          
          pylab.legend(loc=loc)
        
        pylab.subplots_adjust(right=0.98)
        pylab.subplots_adjust(left=0.03)
        pylab.subplots_adjust(top=0.94)
        
        if infolines: self.showInfo()
        
        if save:
           pylab.savefig(save)
           if not movie: self.generateTexTable(save)
        if not frame and not movie:
            pylab.show()
    
    def subplot_2d(self, v, ts, extent_mean=None, extent_var=None, frame=False, infolines=True, save=None, topography=False, cut_slice=None, movie=False, log_var=False, contour=False, poster=False):
        
        if cut_slice:
          self.subplot_2d_slice (v, ts, extent_mean, extent_var, frame, infolines, save, cut_slice, movie, log_var=log_var)
          return
        
        if infolines:
          if self.i.COMPUTE_VARIANCE:
            pylab.figure(figsize=(14,6))
          else:
            pylab.figure(figsize=(7,6))
        else:
          if self.i.COMPUTE_VARIANCE:
            pylab.figure(figsize=(14,5))
          else:
            pylab.figure(figsize=(7,5))
        
        # mean
        s = {}
        if extent_mean:
          s['vmin'] = extent_mean[0]
          s['vmax'] = extent_mean[1]
        
        d = self.var(v, ts, 'mean')
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H and (not 'stats' in self.opts or self.opts['stats'] == 'mean-var'):
          d += self.var(sw.B, ts, "mean")
        
        if int(self.vars['L']) != 0 or int(self.vars['ML']) != 1:
          title_prefix = "mean of "
        else:
          title_prefix = ""
        
        if self.i.COMPUTE_VARIANCE: pylab.subplot(121)
        
        if contour:
          pylab.contour(numpy.transpose(d[:,:,0]), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
        else:
          pylab.imshow(numpy.transpose(d[:,:,0]), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
        if 'stats' in self.opts and self.opts['stats'] == 'treshold-sw':
          pylab.title('probability of E at t=%.2f' % self.times[ts])
        else:
          pylab.title(title_prefix + v.name + ' at t=%.2f' % self.times[ts])
        if poster: pylab.axis('off')
        
        # MUST be before all 'pylab.colorbar()' or otherwise plot is distorted (any idea why?)
        if infolines:
          self.showInfo()
          pylab.subplots_adjust(right=0.96)
          pylab.subplots_adjust(left=0.00)
          pylab.subplots_adjust(top=0.94)
          pylab.subplots_adjust(wspace=0.05)
        else:
          pylab.subplots_adjust(right=0.96)
          pylab.subplots_adjust(left=0.00)
          pylab.subplots_adjust(top=0.92)
          pylab.subplots_adjust(wspace=0.05)
        
        pylab.colorbar()
        if 'equation' in self.opts and self.opts['equation'] == 'sw':
          pylab.winter()
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and topography and not contour:
          self.plotTopography_2d((0,self.f.MAXX,0,self.f.MAXY), ts, "mean")
        
        # variance
        if not 'COMPUTE_VARIANCE' in self.vars or int(self.vars['COMPUTE_VARIANCE']) == 1:
          s = {}
          if extent_var:
            if log_var:
              s['vmin'] = numpy.log10( numpy.max( log10_for_variance_cutoff, extent_var[0] ) )
              s['vmax'] = numpy.log10( extent_var[1] )
            else:
              s['vmin'] = extent_var[0]
              s['vmax'] = extent_var[1]
          
          d = self.var(v, ts, 'variance')
          if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H and (not 'stats' in self.opts or self.opts['stats'] == 'mean-var'):
            d = numpy.fabs(d - self.var(sw.B, ts, "variance"))
          
          pylab.subplot(122)
          
          '''
          if extent_var:
            if contour:
              pylab.contour(numpy.transpose(d[:,:,0]), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
            else:
              pylab.imshow(numpy.transpose(d[:,:,0]), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
            pylab.title('variance' + ' of ' + v.name + ' at t=%.2f' % self.times[ts])
          '''
          
          if log_var:
            var_cutoff = numpy.transpose(d[:,:,0])
            var_cutoff = numpy.where(var_cutoff < log10_for_variance_cutoff, log10_for_variance_cutoff, var_cutoff)
            if contour:
              pylab.contour(numpy.log10(var_cutoff), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
            else:
              pylab.imshow(numpy.log10(var_cutoff), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
            pylab.title('log10(variance)' + ' of ' + v.name + ' at t=%.2f' % self.times[ts])
          
          else:
            if contour:
              pylab.contour(numpy.transpose(d[:,:,0]), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
            else:
              pylab.imshow(numpy.transpose(d[:,:,0]), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
            pylab.title('variance' + ' of ' + v.name + ' at t=%.2f' % self.times[ts])
          
          pylab.colorbar()
          if 'equation' in self.opts and self.opts['equation'] == 'sw':
            pylab.winter()
          
          if 'equation' in self.opts and self.opts['equation'] == 'sw' and topography and not contour:
            self.plotTopography_2d((0,self.f.MAXX,0,self.f.MAXY), ts, "variance")
          
          if poster: pylab.axis('off')
        
        if save:
          pylab.savefig(save)
          if not movie: self.generateTexTable(save)
        if not frame and not movie:
          pylab.show()
    
    def subplot_2d_slice(self, v, ts, extent_mean, extent_var, frame, infolines, save, cut_slice, movie, log_var=False):
        
        if infolines:
          pylab.figure(figsize=(14,6))
        else:
          pylab.figure(figsize=(14,5))
        
        xr = []
        slice_at = 0
        if cut_slice['dir'] == 'X':
          xr = numpy.arange(self.nx)*(self.f.MAXX/self.nx)+0.5*self.f.MAXX/self.nx
          slice_at = round(float(cut_slice['pos']) / self.f.MAXX * self.nx)
        else:
          xr = numpy.arange(self.ny)*(self.f.MAXY/self.ny)+0.5*self.f.MAXY/self.ny
          slice_at = round(float(cut_slice['pos']) / self.f.MAXY * self.ny)
        
        d = self.var(v, ts, 'mean')
        
        if int(self.vars['L']) != 0 or int(self.vars['ML']) != 1:
          title_prefix = "mean of "
        else:
          title_prefix = ""
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H and (not 'stats' in self.opts or self.opts['stats'] == 'mean-var'):
          d += self.var(sw.B, ts, "mean")
        
        pylab.subplot(121)
        if cut_slice['dir'] == 'X':
          pylab.plot(xr, d[slice_at,:,0], label=self.name)
          pylab.title(title_prefix + v.name + ' at t=%.2f, x=%.2f' % (self.times[ts], float(cut_slice['pos'])))
        else:
          pylab.plot(xr, d[:,slice_at,0], label=self.name)
          pylab.title(title_prefix + v.name + ' at t=%.2f, y=%.2f' % (self.times[ts], float(cut_slice['pos'])))
        pylab.legend(loc='upper right')
        
        if extent_mean:
            pylab.ylim(*extent_mean)
        
        if not 'COMPUTE_VARIANCE' in self.vars or int(self.vars['COMPUTE_VARIANCE']) == 1:
          d = self.var(v, ts, 'variance')
          if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H and (not 'stats' in self.opts or self.opts['stats'] == 'mean-var'):
            d = numpy.fabs(d - self.var(sw.B, ts, "variance"))
          pylab.subplot(122)
          if cut_slice['dir'] == 'X':
            if not log_var:
              pylab.plot(xr, d[slice_at,:,0], label=self.name)
              if not 'COMPUTE_KURTOSIS' in self.vars or int(self.vars['COMPUTE_KURTOSIS']) == 0:
                pylab.title('variance' + ' of ' + v.name + ' at t=%.2f, x=%.2f' % (self.times[ts], float(cut_slice['pos'])))
              else:
                pylab.title('4th centered moment' + ' of ' + v.name + ' t=%.2f, x=%.2f' % (self.times[ts], float(cut_slice['pos'])))
            else:
              var_cutoff = d[slice_at,:,0]
              var_cutoff = numpy.where(var_cutoff < log10_for_variance_cutoff, log10_for_variance_cutoff, var_cutoff)
              pylab.plot(xr, numpy.log10(var_cutoff), label=self.name)
              if not 'COMPUTE_KURTOSIS' in self.vars or int(self.vars['COMPUTE_KURTOSIS']) == 0:
                pylab.title('log10(variance)' + ' of ' + v.name + ' at t=%.2f, x=%.2f' % (self.times[ts], float(cut_slice['pos'])))
              else:
                pylab.title('log10(kurtosis)' + ' of ' + v.name + ' t=%.2f, x=%.2f' % (self.times[ts], float(cut_slice['pos'])))
          else:
            if not log_var:
              pylab.plot(xr, d[:,slice_at,0], label=self.name)
              if not 'COMPUTE_KURTOSIS' in self.vars or int(self.vars['COMPUTE_KURTOSIS']) == 0:
                pylab.title('variance' + ' of ' + v.name + ' at t=%.2f, y=%.2f' % (self.times[ts], float(cut_slice['pos'])))
              else:
                pylab.title('4th centered moment' + ' of ' + v.name + ' t=%.2f, y=%.2f' % (self.times[ts], float(cut_slice['pos'])))
            else:
              var_cutoff = d[:,slice_at,0]
              var_cutoff = numpy.where(var_cutoff < log10_for_variance_cutoff, log10_for_variance_cutoff, var_cutoff)
              pylab.plot(xr, numpy.log10(var_cutoff), label=self.name)
              if not 'COMPUTE_KURTOSIS' in self.vars or int(self.vars['COMPUTE_KURTOSIS']) == 0:
                pylab.title('log10(variance)' + ' of ' + v.name + ' at t=%.2f, y=%.2f' % (self.times[ts], float(cut_slice['pos'])))
              else:
                pylab.title('log10(kurtosis)' + ' of ' + v.name + ' t=%.2f, y=%.2f' % (self.times[ts], float(cut_slice['pos'])))
          if extent_var:
              pylab.ylim(*extent_var)
          pylab.legend(loc='upper right')
        
        pylab.subplots_adjust(right=0.97)
        pylab.subplots_adjust(left=0.07)
        pylab.subplots_adjust(top=0.94)
        pylab.subplots_adjust(wspace=0.30)
        
        if infolines: self.showInfo()
        
        if save:
            pylab.savefig(save)
            if not movie: self.generateTexTable(save)
        if not frame and not movie:
            pylab.show()
    
    def getExtents (self, v, ts, margin=0.0):
      mean = self.var(v, ts, "mean")
      if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H and (not 'stats' in self.opts or self.opts['stats'] == 'mean-var'):
        mean += self.var(sw.B, ts, "mean")
      
      low  = numpy.min(mean)
      high = numpy.max(mean)
      diff = high - low
      
      low  -= diff * margin
      high += diff * margin
      
      ext_mean = (low, high)
      
      if not 'COMPUTE_VARIANCE' in self.vars or int(self.vars['COMPUTE_VARIANCE']) == 1:
        var  = self.var(v, ts, "variance")
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H and (not 'stats' in self.opts or self.opts['stats'] == 'mean-var'):
          var = numpy.fabs(var - self.var(sw.B, ts, "variance"))
        
        low  = numpy.min(var)
        high = numpy.max(var)
        diff = high - low
        
        low  -= diff * margin
        high += diff * margin
      
      ext_var = (low, high)
      
      return [ext_mean, ext_var]
    
    def movie(self, v, slowdown=5, infolines=True, last_extent=False, cut_slice=None, topography=False, log_var=False):
        
        margin = 0.1
        
        # for 2d plots, colorbars also indicate both extrema, hence should not be modified
        if self.i.NY != 1:
          margin = 0.0
        
        if last_extent:
          [ext_mean, ext_var] = self.getExtents(v, len(self.times)-1, margin)
        else:
          [ext_mean, ext_var] = self.getExtents(v, 0, margin)
          
          for ts in range(1,len(self.times)):
              
              [ext_mean_tmp, ext_var_tmp] = self.getExtents(v, ts, margin)
              
              ext_mean = (min(ext_mean_tmp[0], ext_mean[0]), max(ext_mean_tmp[1], ext_mean[1]))
              ext_var  = (min(ext_var_tmp [0], ext_var [0]), max(ext_var_tmp [1], ext_var [1]))
        
        for ts in range(len(self.times)):
            self.subplot(v, False, ts, ext_mean, ext_var, infolines, topography=topography, movie=True, cut_slice=cut_slice, log_var=log_var)
            pylab.savefig('movie_%04d.png' % ts)
            pylab.close()
        
        fps = round ( int(self.vars['NSAVES']) / float(self.vars['MAXT']) )
        
        os.system('mencoder "mf://*.png" -mf fps=%d -o %s_%s.mp4 -ovc lavc -lavcopts vcodec=mpeg4:vbitrate=800' % (fps/slowdown, self.movief, v.name))
        #os.system('mencoder "mf://*.png" -mf fps=%d -o %s.avi -ovc lavc -lavcopts vcodec=msmpeg4v2:vbitrate=800' % (fps/slowdown, self.movief))
        
        os.system('rm movie_*.png')
    
    def matlab(self, v, ts=None, path=''):
        if ts is None: ts = len(self.times)-1
        data = {}
        
        data['x'] = numpy.arange(self.nx)*(self.f.MAXX/self.nx)+0.5*self.f.MAXX/self.nx
        
        if self.ny == 1:
          data['mean'] = self.var(v, ts, 'mean')[:,0,0]
          if int(self.vars['COMPUTE_VARIANCE']) == 1:
            data['variance'] = self.var(v, ts, 'variance')[:,0,0]
        
        elif self.nz == 1:
          data['y'] = numpy.arange(self.ny)*(self.f.MAXY/self.ny)+0.5*self.f.MAXY/self.ny
          data['mean'] = self.var(v, ts, 'mean')[:,:,0]
          if int(self.vars['COMPUTE_VARIANCE']) == 1:
            data['variance'] = self.var(v, ts, 'variance')[:,:,0]
        
        else:
          data['z'] = numpy.arange(self.nz)*(self.f.MAXz/self.nz)+0.5*self.f.MAXZ/self.nz
          data['mean'] = self.var(v, ts, 'mean')[:,:,:]
          if int(self.vars['COMPUTE_VARIANCE']) == 1:
            data['variance'] = self.var(v, ts, 'variance')[:,:,:]
        
        if 'wall_runtime_total' in self.runtime:
          data['time'] = self.runtime['wall_runtime_total']
        else:
          data['time'] = self.runtime['clock_runtime']
        
        from scipy.io import savemat
        savemat(path + self.name + '_' + v.name + '.mat', data)
    
    def plot_histograms(self, infolines=True, frame=False, hist_name='value', bins=50, save=None, loc='upper right'):
        
        if not 'HISTOGRAMS' in self.vars or self.vars['HISTOGRAMS'] != '1':
          raise Exception ("ERROR: Histogram data not avaiglable! Use HISTOGRAMS=1 when compiling.")
        
        # computing positioning of the subplots
        if self.i.L == 0:
            num_plots_x = 1
            num_plots_y = 1
        else:
            num_plots_x = 2
            num_plots_y = numpy.floor ( self.i.L/2.0 + 1 )
        
        if infolines:
          pylab.figure(figsize=(7*num_plots_x,3*num_plots_y+1))
        else:
          pylab.figure(figsize=(7*num_plots_x,3*num_plots_y))
        
        for level in xrange(0,self.i.L+1):
          pylab.subplot(num_plots_y, num_plots_x, level+1)
          log = len(self.hist['hist_data'][level]) >= 100
          pylab.hist(self.hist['hist_data'][level], label='level %d' % level, bins=bins, log=log)
          pylab.ylabel('count')
          pylab.legend(loc=loc)
          #pylab.xlabel(hist_name)
        
        pylab.subplots_adjust(top=0.96)
        pylab.subplots_adjust(right=0.96)
        pylab.subplots_adjust(left=0.06)
        
        if infolines:
          self.showInfo()
          pylab.subplots_adjust(bottom=0.15)
        else:
          pylab.subplots_adjust(bottom=0.05)
        
        if save:
            pylab.savefig(save)
            self.generateTexTable(save)
        if not frame:
            pylab.show()

print ("")

try:
  r = Run();
  print (" :: Default run loaded as \"r\".")
except:
  print (" :: Default run not found!")

try:
  mc = Run('MC');
  print (" :: MC run loaded as \"mc\".")
except:
  print (" :: MC run not found!")

try:
  mlmc = Run('MLMC');
  print (" :: MLMC run loaded as \"mlmc\".")
except:
  print (" :: MLMC run not found!")

print ("")

def tables (var):
  table([mc, mlmc, mlmcd, mc2, mlmc2, mlmc2d], var, None, "%12.1e", None, None, ref)

# helpers

# path for local saving of figures with descriptive TeX files
IMG_PATH = 'img'

def getFileName (refr, suffix=None, file_suffix='pdf'):

  if not os.path.exists(IMG_PATH):
    os.mkdir(IMG_PATH)
    os.system('svn add ' + IMG_PATH)
  if 'equation' in refr.opts:
    name = os.path.join(IMG_PATH,refr.opts['equation'])
    name += '-' + refr.opts['model']
  else:
    name = os.path.join(IMG_PATH,refr.opts['model'])
  if 'equation' in refr.opts and refr.opts['equation'] == 'sw':
    name += '-' + refr.opts['topography']
  if 'flux' in refr.opts and refr.opts['flux'] != 'det':
    name += '-' + refr.opts['flux']
  if 'equation' in refr.opts and refr.opts['equation'] == 'wave':
    name += '-' + refr.opts['material']
  if suffix:
    name += '_' + suffix
  if file_suffix:
    name += '.' + file_suffix
  else:
    name += '.pdf'
  return name

def svnup():
  if not os.path.exists(IMG_PATH):
    os.system('svn up')

def svnadd():
  os.system('svn add -q ' + os.path.join(IMG_PATH, '*'))
