
from enthought.mayavi.scripts import mayavi2
mayavi2.standalone(globals())

from enthought.mayavi import mlab

#mlab.options.backend = 'envisage'

import numpy
import os

from variables import *

blue  = (0.0,0.0,1.0)
brown = (0.4,0.2,0.0)
black = (0.0,0.0,0.0)
red   = (1.0,0.0,0.0)
white = (1.0,1.0,1.0)

class Run (object):
    def __init__(self, name=None):
        if name:
            self.outputf = 'output_'+name+'.dat'
            self.varf = 'var_output_'+name+'.dat'
            self.timesf = 'times_'+name+'.py'
            self.configf = 'config_'+name+'.py'
            self.runtimef = 'runtime_'+name+'.py'
            self.movief = 'movie_'+name
            self.name = name
        else:
            self.outputf = 'output.dat'
            self.varf = 'var_output.dat'
            self.timesf = 'times.py'
            self.configf = 'config.py'
            self.runtimef = 'runtime.py'
            self.movief = 'movie'
            self.name = 'run'
        times = {}
        execfile(self.timesf, globals(), times)
        self.times = times['time']
        config = {}
        execfile(self.configf, globals(), config)
        self.opts = config['opts']
        self.vars = config['used']
        self.runtime = {}
        execfile(self.runtimef, globals(), self.runtime)
        self.i = Config(self, int)
        self.f = Config(self, float)
        self.nx = self.i.NX
        self.ny = self.i.NY
        self.nz = self.i.NZ
        self.dxyz = (self.f.MAXX/self.nx, self.f.MAXY/self.ny, self.f.MAXZ/self.nz)
        
        # extracting the data (infolines) to be displayed on the plots
        self.info_opts = self.opts
        self.info_vars = config['vars']
        if 'RUNTIME_FILE' in self.info_vars: del self.info_vars ['RUNTIME_FILE']
        if 'LOG_FILE'     in self.info_vars: del self.info_vars ['LOG_FILE']
        if 'TIME_FILE'    in self.info_vars: del self.info_vars ['TIME_FILE']
        if 'DATA_FILE'    in self.info_vars: del self.info_vars ['DATA_FILE']
        if 'DATA_PATH'    in self.info_vars: del self.info_vars ['DATA_PATH']
        
        # detecting number of variables from equation type
        if 'equation' in self.opts:
          self.NVARS = get_NVARS (self.opts['equation'])
        else:
          self.NVARS = 16
    
    def load(self, vi, ts, case='mean'):
        if ts > len(self.times):
            raise Exception ("Timestep out of bounds")
        if case == "mean":
            f = open(self.outputf, 'rb')
        else: # case="variance"
            f = open(self.varf, 'rb')
        n = self.nx*self.ny*self.nz
        f.seek(8*n*(vi + self.NVARS*ts))
        fd = numpy.fromfile(f, dtype=numpy.float64, count=n)
        f.close()
        fd.shape = (self.nx, self.ny, self.nz)
        return fd
    
    def var(self, v, ts=None):
        if ts is None: ts = len(self.times)-1
        name = "%s %s %g"%(self.name, v.name, self.times[ts])
        src = v.load(self, ts, name)
    
    def showInfo(self):
      
      string_opts = ''.join([' %s:%s,' % x for x in sorted(self.info_opts.items())])
      string_vars = ''.join([' %s=%s' % x for x in sorted(self.info_vars.items())])
      
      text  = 'OPTS:' + string_opts[0:-1] + '\n'
      text += 'VARS:' + string_vars + '\n'
      
      if 'wall_runtime' in self.runtime:
        runtime = self.runtime['wall_runtime']
      else:
        runtime = self.runtime['clock_runtime']
      
      import datetime
      runtime = str(datetime.timedelta(seconds=round(runtime)))
      
      text += 'INFO: ' + 'cores: ' + str(self.runtime['nodes']) + ', ' + 'runtime: ' + runtime
      
      if 'mpi_scaling' in self.runtime:
        text += ', ' + 'efficiency: ' + str(round(1000 * self.runtime['mpi_scaling']) / 10.0) + r'%'
      
      mlab.text(0.01, 0.01, text, color=black, line_width=2.0, width=1.0, name='infolines')
    
    def generateTexTable(self, save=None):
      
      # config
      
      keys     =  ['L',    'ML',     'grid_size', 'CFL', 'cores', 'runtime']
      captions =  [r'$L$', r'$M_L$', 'grid size', 'CFL', 'cores', 'runtime']
      if 'mpi_scaling' in self.runtime:
        keys     += ['efficiency']
        captions += ['efficiency']
      
      # aggregation of information
      
      opts = {}
      opts.update(self.opts)
      opts.update(self.vars)
      
      grid_size = str(opts['NX'])
      if self.ny != 1:
        grid_size += 'x' + str(opts['NY'])
      if self.nz != 1:
        grid_size += 'x' + str(opts['NZ'])
      opts.update( { 'grid_size' : grid_size } )
      
      opts.update( { 'cores' : str(self.runtime['nodes']) } )
      
      if 'wall_runtime' in self.runtime:
        runtime = self.runtime['wall_runtime']
      else:
        runtime = self.runtime['clock_runtime']
      
      import datetime
      runtime = str(datetime.timedelta(seconds=round(runtime)))
      
      opts.update( { 'runtime' : str(runtime) } )
      
      if 'mpi_scaling' in self.runtime:
        opts.update( { 'efficiency' : str(round(1000 * self.runtime['mpi_scaling']) / 10.0) + r'\%' } )
      
      # TeX code generation
      
      columns =  '|' + 'c|' * len(keys)
      
      text =  '\n'
      text += r'\begin{tabular}{%s}' % columns + '\n'
      
      text += r'\hline' + '\n'
      text += (r'%s & ' * len(captions))[0:-2] % tuple(captions) + r'\\' + '\n'
      text += r'\hline' + '\n'
      text += (r'%s & ' * len(keys))[0:-2] % tuple([opts[key] for key in keys]) + r'\\' + '\n'
      text += r'\hline' + '\n'
      
      text += r'\end{tabular}' + '\n'
      
      # saving
      
      f = open(save[0:-4] + '.tex', 'w')
      f.write(text)
      f.close()
    
    def plot(self, ts=None, infolines=True, frame=False, animate=False, view={'azimuth':270, 'elevation':75}, save=None):
        
        print 'Not implemented'
    
    def plotSW(self, ts=None, infolines=True, frame=False, animate=False, view={'azimuth':270, 'elevation':75}, save=None):
        
        if save:
          self.generateTexTable(save)
        
        if ts is None: ts = len(self.times)-1
        
        xr = numpy.arange(self.nx)*(self.f.MAXX/self.nx)
        yr = numpy.arange(self.ny)*(self.f.MAXY/self.ny)
        
        mlab.figure(size=(1024, 1024))
        
        mean = self.load(0, ts, 'mean')
        b    = self.load(8, ts, 'mean')
        mean += b
        
        ms_mean = mlab.surf(xr, yr, mean[:,:,0], color=blue, opacity=0.5, name='mean').mlab_source
        ms_b    = mlab.surf(xr, yr, b[:,:,0],                             name='topography').mlab_source
        
        if int(self.vars['COMPUTE_VARIANCE']) == 1:
          var = self.load(0, ts, 'variance')
          b_var = self.load(8, ts, 'variance')
          b_std = numpy.sqrt(b_var)
          var  -= b_var
          std = numpy.sqrt(numpy.fabs(var))
          
          ms_mean_p_std = mlab.surf(xr, yr, (mean+std)[:,:,0], colormap='cool', opacity=0.2, name='mean+std').mlab_source
          ms_b_std      = mlab.surf(xr, yr, (b+b_std)[:,:,0],                   opacity=0.2, name='topography+std').mlab_source
        
        ms_title = mlab.title('b and h+b at t=%.2f' % self.times[ts], height=0.95, size=0.3, color=black)
        
        mlab.xlabel('x')
        mlab.ylabel('y')
        mlab.zlabel('')
        
        if int(self.vars['COMPUTE_VARIANCE']) == 1:
          mlab.axes(color=black, extent=(0, self.f.MAXX, 0, self.f.MAXY, numpy.min(b), numpy.max(mean+std)))
        else:
          mlab.axes(color=black, extent=(0, self.f.MAXX, 0, self.f.MAXY, numpy.min(b), numpy.max(mean)))
        
        dist = mlab.view()[2]
        mlab.view(view['azimuth'], view['elevation'], distance=dist-0.4)
        #mlab.yaw(4)
        
        if infolines: self.showInfo()
        
        if animate:
            
            sc = mlab.gcf()
            
            # animate thoughout the remaining snapshots
            for ts in range(1, len(self.times)):
              
              # start performance optimization
              mlab.gcf().scene.disable_render = True
              
              # compute data
              
              mean = self.load(0, ts, 'mean')
              var = self.load(0, ts, 'variance')
              
              mean += b
              var  -= b_var
              
              std = numpy.sqrt(numpy.fabs(var))
              
              # update scene
              
              ms_mean.scalars = mean[:,:,0]
              
              ms_mean_p_std.scalars = (mean + std)[:,:,0]
              
              ms_title.text = 'b and h+b at t=%.2f' % self.times[ts]
              
              # end performance optimization
              mlab.gcf().scene.disable_render = False
        
        if not frame: mlab.show()
    
    def plotTopography(self, infolines=True, frame=False, view={'azimuth':270, 'elevation':75}, title=True, save=None):
        
        if save:
          self.generateTexTable(save)
        
        ts = len(self.times)-1
        
        xr = numpy.arange(self.nx)*(self.f.MAXX/self.nx)
        yr = numpy.arange(self.ny)*(self.f.MAXY/self.ny)
        
        mlab.figure(size=(1024,800))
        
        b = self.load(8, ts, 'mean')
        
        ms_b = mlab.surf(xr, yr, b[:,:,0], name='topography')
        
        if int(self.vars['COMPUTE_VARIANCE']) == 1:
          b_var = self.load(8, ts, 'variance')
          b_std = numpy.sqrt(b_var)
          ms_b_std = mlab.surf(xr, yr, (b+b_std)[:,:,0], opacity=0.2, name='topography+std')
        
        if title: ms_title = mlab.title('bottom topography', height=0.95, size=0.3, color=black)
        
        mlab.xlabel('x')
        mlab.ylabel('y')
        mlab.zlabel('')
        
        if int(self.vars['COMPUTE_VARIANCE']) == 1:
          mlab.axes(color=black, extent=(0, self.f.MAXX, 0, self.f.MAXY, numpy.min(b), numpy.max(b+b_std)))
        else:
          mlab.axes(color=black, extent=(0, self.f.MAXX, 0, self.f.MAXY, numpy.min(b), numpy.max(b)))
        
        dist = mlab.view()[2]
        mlab.view(view['azimuth'], view['elevation'], distance=dist-0.8)
        #mlab.yaw(4)
        
        if infolines: self.showInfo()
        
        if not frame: mlab.show()
    
    def movie(self, slowdown=5, infolines=True, topography=True):
        
        for ts in range(len(self.times)):
            self.plotSW(ts, infolines, frame=True, animate=False)
            mlab.savefig('movie_%04d.png' % ts, size=(1024,768))
            mlab.close()
        
        fps = round ( int(self.vars['NSAVES']) / float(self.vars['MAXT']) )
        
        os.system('mencoder "mf://*.png" -mf fps=%d -o %s_%s.mp4 -ovc lavc -lavcopts vcodec=mpeg4:vbitrate=800' % (fps/slowdown, self.movief, 'h+b'))
        #os.system('mencoder "mf://*.png" -mf fps=%d -o %s.avi -ovc lavc -lavcopts vcodec=msmpeg4v2:vbitrate=800' % (fps/slowdown, self.movief))
        
        os.system('rm movie_*.png')

print ("")

try:
  r = Run();
  print (" :: Default run loaded as \"r\".")
except:
  print (" :: Default run not found!")

print ("")
