
/* EQUATION SPECIFIC FVM routines for FLUX
 * Karhunen-Loeve expansion of flux (Burgers' equation):
 * f(u) = u^2/2 + Karhunen-Loeve expansion
 */

///::title:: Karhunen-Loeve expansion (Burgers' 1D)
///::vars:: KL=10 KL_DECAY=2.0 KL_SCALING=0.01
///::consts:: M_UNIFORM=KL M_NORMAL=0

#include "equation.h"
#include "chaos.h"

// computes Karhunen-Loeve expansion
real calc_KL_flux (real u) {
  real value = u*u/2.0;
  for (int m=1; m<=KL; m++)
    value += KL_SCALING * pow(m,-KL_DECAY) * sin(m*pi*u) * (2*chaos.m_uniform[m-1] - 1);
  return value;
}

real calc_max_speed (const ConservedVars &CV, const XtraVars &XV, int d)
{
  real value = CV.u;
  for (int m=1; m<=KL; m++)
    value += KL_SCALING * pow(m,-KL_DECAY) * m*pi*cos(m*pi*CV.u) * (2*chaos.m_uniform[m-1] - 1);
  return fabs(value);
}

void calc_point_flux(ConservedVars &F, const AllVariables &V, int d)
{
  F.u = calc_KL_flux(V.U.u);
}

void calc_xtra(XtraVars &Xijk, const ConservedVars &Uijk, int i, int j, int k)
{
  Xijk.E = calc_KL_flux(Uijk.u);
}

void calc_all_vars(AllVariables &V, const PrimitiveVars &P)
{
  P.toConservedVars(V.U);
  V.XU.E = calc_KL_flux(P.u);
}
