
/* EQUATION SPECIFIC FVM routines for FLUX
 * Uniformly perturbed flux (Burgers' equation):
 * fx(u) = u^px/px, fy(u) = u^py/py, fz(u) = u^pz/pz, px,py,pz ~ U (PMIN, PMAX), independent
 */

///::title:: Uniformly perturbed (Burgers' 1D and 2D)
///::consts:: M_UNIFORM=3 M_NORMAL=0
///::vars:: PMIN=1.0 PMAX=3.0

#include "equation.h"
#include "chaos.h"

real calc_max_speed (const ConservedVars &CV, const XtraVars &XV, int d)
{
  return pow(fabs(CV.u), PMIN + (PMAX-PMIN)*chaos.m_uniform[d] - 1);
}

real p;

void calc_point_flux(ConservedVars &F, const AllVariables &V, int d)
{
  p = PMIN + (PMAX-PMIN)*chaos.m_uniform[d];
  F.u = pow(fabs(V.U.u), p) / p;
}

void calc_xtra(XtraVars &Xijk, const ConservedVars &Uijk, int i, int j, int k)
{
  Xijk.E = pow(fabs(Uijk.u), 2) / 2;
}

void calc_all_vars(AllVariables &V, const PrimitiveVars &P)
{
  P.toConservedVars(V.U);
  V.XU.E = pow(fabs(P.u), 2) / 2;
}
