
/* EQUATION SPECIFIC FVM routines for FLUX
 * Standard deterministic flux (Shallow Water equations)
 */

///::title:: Deterministic (Shallow Water Equations)

#include "equation.h"
#include "solver-util-ec.h"

void calc_point_flux(ConservedVars &F, const AllVariables &V, int d)
{
  if (d != 2) {
    F.h       = V.U.h * V.XU.u[d];
    F.hu[d]   = V.U.h * sqr(V.XU.u[d]) + GRAVITY * sqr(V.U.h) / 2.0;
    F.hu[1-d] = V.U.h * V.XU.u[d] * V.XU.u[1-d];
  }
}

void calc_xtra(XtraVars &Xijk, const ConservedVars &Uijk, int i, int j, int k)
{
  Xijk.u = Uijk.hu / Uijk.h;
  Xijk.E = ( Uijk.h * (Xijk.u * Xijk.u) + GRAVITY * sqr(Uijk.h) ) / 2.0;
  Xijk.b = Xtra[i][j][k].b;
}

void calc_all_vars(AllVariables &V, const PrimitiveVars &P)
{
  P.toConservedVars(V.U);
  V.XU.u = P.u;
  V.XU.E = ( P.h * (P.u * P.u) + GRAVITY * sqr(P.h) ) / 2.0;
}
