
/* EQUATION SPECIFIC FVM routines for FLUX
 * Standard deterministic flux (Wave equation, 2nd type)
 */

///::title:: Deterministic (Wave equation)
///::vars:: ADVECTX=0.0 ADVECTY=0.0 ADVECTZ=0.0
////::macro:: NON_CONSERVATIVE_FLUX_HOOK @  F.u *= Xtra[i][j][k].c;

#include "equation.h"
#include "material.h"

void calc_point_flux(ConservedVars &F, const AllVariables &V, int d)
{ 
  F.p    = - V.XU.c * V.U.u[d];
  F.u[d] = - V.U.p;
  //F.p    = - V.U.u[d];
  //F.u[d] = - V.U.p;
  if (d == X) F.u -= ADVECTX*V.U.u;
  if (d == Y) F.u -= ADVECTY*V.U.u;
  if (d == Z) F.u -= ADVECTZ*V.U.u;
}

void calc_xtra(XtraVars &Xijk, const ConservedVars &Uijk, int i, int j, int k)
{
  Xijk.c = Xtra[i][j][k].c;
  Xijk.E = sqr( Uijk.p );
}

void calc_all_vars(AllVariables &V, const PrimitiveVars &P)
{
  P.toConservedVars(V.U);
  V.XU.E = sqr( P.p );
}
