
// === Routines for random lognormal material coefficient generation using FFT

#include "numtypes.h"

#include <fftw3.h>

// kernel size (depends on the value of UPSCALE)
extern int NXl_kernel, NYl_kernel, NZl_kernel;

// material coefficient buffer
extern volume <real, 0> c_buffer;

// === User-defined routines (defined in model_*.cpp files)

// defines a stationary covariance kernel (for periodic domains, kernel depends not only on distance, but also on direction)
real material_covariance_kernel (const vector &x1, const vector &x2);

// === Non-user-defined routines (defined in material.cpp file)

// initializes correlation kernel, computes its FFT transform
void material_fft_init ();

// draws one realization of the random lognormal material coefficient
void material_fft_draw_sample ();

// deallocates memory, destructs instances
void material_fft_exit();

// === Helpers

// performs appropriate multi-d real to complex FFT transform
void material_fft_r2c (real *in, fftw_complex *out);

// performs appropriate multi-d complex to real FFT transform
void material_fft_c2r (fftw_complex *in, real *out);

// return an upscaled kernel 'in' by the required number of 'levels'
volume <real, 0> * material_fft_upscale_kernel (volume <real, 0> *in, int levels);

// averages volume 'v' to a coarser mesh resolution
// averages are stored in the first half of the index ranges
void material_fft_average_volume (volume <real, 0> &v);

// returns location of the multi-index in the linear memory model
int getLoc (int i, int j, int k);
