
#include "material.h"

#include "equation.h"
#include "quadrature.h"
#include "multi.h"

real c (const int &i, const int &j, const int &k) {
  
  #if BUFFERED_MATERIAL
  
  // copy from buffer
  return c_buffer[mod(i + NXl*multix, NXl_FULL)][mod(j + NYl*multiy, NYl_FULL)][mod(k + NZl*multiz, NZl_FULL)];
  
  #else
  
  // use quadrature rule
  real vc = 0.0;
  real qc;
  for (int px=0; px<(HAVE_DIR(X) ? QUAD_NUM : 1); px++)
    for (int py=0; py<(HAVE_DIR(Y) ? QUAD_NUM : 1); py++)
      for (int pz=0; pz<(HAVE_DIR(Z) ? QUAD_NUM : 1); pz++) {
        qc = c ( (mod(i + NXl*multix, NXl_FULL) + 0.5 + QUAD_POINTS[px]) * dx[X],
                 (mod(j + NYl*multiy, NYl_FULL) + 0.5 + QUAD_POINTS[py]) * dx[Y],
                 (mod(k + NZl*multiz, NZl_FULL) + 0.5 + QUAD_POINTS[pz]) * dx[Z] );
        if (HAVE_DIR(X)) qc *= QUAD_WEIGHTS[px];
        if (HAVE_DIR(Y)) qc *= QUAD_WEIGHTS[py];
        if (HAVE_DIR(Z)) qc *= QUAD_WEIGHTS[pz];
        vc += qc;
      }
  
  return vc;
  
  #endif
}
