
#include "material.h"

#include "material-fft.h"

///::title:: Log-normal PERIODIC material coefficient using FFT
///::vars:: VARIANCE=1.0 CORRELATION_LENGTH=0.5
///::consts:: BUFFERED_MATERIAL=1
///::consts:: M_UNIFORM=0 M_NORMAL=NX*NY*NZ
///::depends:: material-fft
///::macro:: LEVEL_INIT_HOOK material-fft.h material_fft_init();
///::macro:: LEVEL_EXIT_HOOK material-fft.h material_fft_exit();

void material_c_init () {
  material_fft_draw_sample ();
}

// helper for material_covariance_kernel
real kernel (const real dist) { return sqr(VARIANCE) * exp( -dist / CORRELATION_LENGTH ); }

real material_covariance_kernel (const vector &x1, const vector &x2) {
  
  real numer = 0;
  real denom = 0;
  vector v0; v0.zero();
  vector vp; vp.zero();
  
  for (int i=0; i<(HAVE_DIR(X)?2:1); i++)
    for (int j=0; j<(HAVE_DIR(Y)?2:1); j++)
      for (int k=0; k<(HAVE_DIR(Z)?2:1); k++) {
        vp[X] = i*MAXX;  vp[Y] = j*MAXY;  vp[Z] = k*MAXZ;
        numer += kernel ( dist_euclidean ( v0, (vp-(x1 - x2).abs()).abs() ) );
        denom += kernel ( dist_euclidean ( v0, vp ) );
      }
  
  return numer / denom;
}

/*
Since BUFFERED_MATERIAL=1, routine c() is NOT needed;
all needed routines are in material-fft.h and material-fft.cpp
*/
