
#include "material.h"

#include "chaos.h"

///::title:: Exponential of Karhunen-Loeve expansion (2D version)
///::vars:: KL=10 MATERIAL_SCALING=0.1
///::consts:: M_UNIFORM=0 M_NORMAL=KL*KL KL_DECAY=2.5

real c (real x, real y, real z) {
  
  // Karhunen–Loeve expantion of log(c) consisting of KL basis functions
  real value = 1.0;
  for (int mx=1; mx<=KL; mx++)
    for (int my=1; my<=KL; my++)
      value *= exp( pow(mx,-KL_DECAY/2.0)*pow(my,-KL_DECAY/2.0) * sin(mx*pi*x)*sin(my*pi*y) * chaos.m_normal[(mx-1)*KL+(my-1)] );
  
  return value;
}
