
#include "mlmc.h"
#include "multi.h"

///::title:: MLMC-FVM modelling software for Stochastic systems of Hyperbolic Partial Differential Equations

// Default constants, parameters and file names
///::consts:: ORDER=1 GHOST_SIZE=1 HAVE_GRAVITY=0 INDICATOR=0 ENTROPIC_RECONSTRUCTION=0
///::vars:: SMOOTH_SOLUTION=0 CFL=0.9 NSAVES=1 SAVE0=0 WRITE_DATAFILES=1 PRINT_t=0
///::vars:: DATA_FILE="output.dat" DATA_PATH="" TIME_FILE="times.py" RUNTIME_FILE="runtime.py" HIST_DATA_FILE="hist_data.py"

// Default constants and parameters for models (deterministic and stochastic)
///::consts:: MAXX=1 MAXY=1 MAXZ=1
///::consts:: N_UNIFORM=0 N_NORMAL=0 T_UNIFORM=0 T_NORMAL=0 M_UNIFORM=0 M_NORMAL=0
///::vars:: NX=1 NY=1 NZ=1
///::vars:: ML=1 L=0 DOWNSAMPLE=0 COMPUTE_VARIANCE=1 VARIANCE_FIX=1 COMPUTE_KURTOSIS=0 HISTOGRAMS=0
///::vars:: INITIAL_DATA_OFFSET=0.0 INITIAL_DATA_SCALING=1.0
///::vars:: PRIME_SHIFT=10 RUN_NUMBER=1 SAMPLES_OFFSET=0 NUMERICAL_STABILIZATION=1

// Default parameters for parallel simulations
///::vars:: LOAD_BALANCING_TYPE=1 RAW_SIMULATION_WTIME=1
// Not yet debugged: LOAD_BALANCING_TYPE=DOWNSAMPLE?1:2

// Default hooks
///::macro:: INIT_HOOK @ @
///::macro:: LEVEL_INIT_HOOK @ @
///::macro:: PRE_STEP_HOOK @ @
///::macro:: INITIAL_DATA_HOOK @ @
///::macro:: SOURCE_HOOK @ @
///::macro:: SUB_STEP_HOOK @ @
///::macro:: NON_CONSERVATIVE_FLUX_HOOK @ @
///::macro:: POST_STEP_HOOK @ @
///::macro:: LEVEL_EXIT_HOOK @ @
///::macro:: FINAL_HOOK @ @

// Dependencies
///::depends:: structures-base structures time multi bc chaos saver equation mlmc

int main(int argc, char **argv) {
  
  // initialization
  
  mlmc_init();
  
  multi_init(argc, argv);
  
  INIT_HOOK
  
  run_compatibility_checks();
  multi_run_compatibility_checks();
  display_info();
  
  volumes_init();
  
  start_timer();
  
  // main computations
  
  run_mlmc();
  
  // data post-processing and output
  
  //#pragma acc update host(mean_U[0:NSAVES+SAVE0],mean_X[0:NSAVES+SAVE0])
  
  multi_combine();
  
  stop_timer();
  
  write_datafiles();
  
  // finalization
  
  multi_cleanup();
  
  FINAL_HOOK
  
  multi_finalise();
  
  return 0;
}
