
#include "equation.h"
#include "chaos.h"

///::title:: Initial data with discontinous square and circle
///::vars:: NX=128 NY=128 MAXT=1.0
///::consts:: MAXX=2 MAXY=2
///::bc:: NEUMANN

const real SQUARE_CENTER [2] = {0.5, 0.5};
const real SQUARE_RADIUS     = 0.4;
const real CIRCLE_CENTER [2] = {1.5, 1.5};
const real CIRCLE_RADIUS     = 0.4;

void initial_data (PrimitiveVars &v, real x, real y, real z)
{
  v.u = 0.0;
  // u = 1.0 inside the square
  if ( fabs(x-SQUARE_CENTER[X]) < SQUARE_RADIUS && fabs(y-SQUARE_CENTER[Y]) <= SQUARE_RADIUS )
    v.u = 1.0;
  // u = -1.0 inside the circle
  if ( sqr(x-CIRCLE_CENTER[X]) + sqr(y-CIRCLE_CENTER[Y]) <= sqr(CIRCLE_RADIUS) )
    v.u = -1.0;
}
