
#include "equation.h"

///::title:: Vortex propagating through random medium (2D version)
///::vars:: NX=128 NY=128 MAXT=2.0
///::consts:: GAMMA=(5.0/3.0) MAXX=1 MAXY=1 N_UNIFORM=0 N_NORMAL=0
///::vars:: SMOOTH_SOLUTION=1
///::bc:: REPEAT

void initial_data (PrimitiveVars &v, real x, real y, real z)
{
  real epsilon=0.3, r_c=0.05, alpha=0.204, x_c=0.25, y_c=0.5, M=1.1;
  
  // vortex part
  real tau       = sqrt( sqr(x - x_c) + sqr(y - y_c) ) / r_c;
  real sin_theta = (y - y_c) / (tau * r_c);
  real cos_theta = (x - x_c) / (tau * r_c);
  
  v.u[X] =  epsilon * tau * exp( alpha*(1-sqr(tau)) ) * sin_theta;
  v.u[Y] = -epsilon * tau * exp( alpha*(1-sqr(tau)) ) * cos_theta;
  v.p    = 1.0 -(GAMMA-1) * sqr(epsilon) * exp( 2*alpha*(1-sqr(tau)) ) / ( 4 * alpha * GAMMA );
}
