
#ifndef MULTI_H
#define MULTI_H

#include "numtypes.h"

extern int MULTI[NDIR+1][L+1];          // MULTI[X-Z][l] - number core groups working on the same domain
static const int M = 3;                 // MULTI[M][l] - number core groups working on the same level samples
extern int MULTI_TOTAL;                 // total number of cores
extern int LOWEST_MULTI_LEVEL;          // lowest multi-core level; all lower levels are computed on the same core as this one

extern int LEVEL_BEGIN, LEVEL_END;      // levels to start and to end (might be different from 0 and L for multi:mpi)
extern int MULTIXl, MULTIYl, MULTIZl;   // number of subdomains in a current level (in each direction)
extern int MULTINl;                     // number of subdomains in a current level (total)
extern int MULTIMl;                     // number core groups working on the same level samples
extern int ACTIVE_MULTIMl;              // number core groups working on the same downsampled level samples
extern long_int MULTIM_PARTITION_SIZE;  // numbe of samples per one sampler

extern int multix, multiy, multiz, multi_main;                          // sub-domain index with communicator's topology
extern int multi_domain_root, multi_samplers_root, multi_level_root;    // bools to indicate roots of various communicators
extern int multi_rank, multi_rank_samplers;                             // ranks within various communicators

// === parallelization-specific routines -- implemented in corresponding multi_*.cpp file

// computes various parameters for parallelization
// should be executed after mlmc_init()
void multi_init(int argc, char **argv);

// mainly, handles assumption s < (d+1)/2
void multi_run_compatibility_checks();

// sets up the mesh-size independent parameters for a particular level (communicators, etc.)
// should be executed after level_init()
void multi_level_init();

// sets up the mesh-size dependent parameters for a particular level (inernal boundary buffer, etc.)
// should be executed after multi_level_init()
// should be re-executed after every re-use of level_init()
void multi_level_setup();

// returns minimal timestep throughout the entire domain
real multi_dt(real dt);

// applies boundary conditions at sub-domain interfaces
void multi_bc();

// combines samples within sampler communicators
void multi_combine_samples();

// combines levels within level communicators
void multi_combine_levels();

// combines historgram data
void multi_combine_hist_data();

// clean up routines (deallocating memory, etc.)
void multi_cleanup();

// finalizing calls
void multi_finalise();

// timers
void start_timer();
void stop_timer();
void print_runtime (float runtime);

#endif
