
#include <sstream>

#include "saver.h"

#include "mlmc.h"
#include "multi.h"

Saver::Saver(std::string quantity)
{
  
  #if WRITE_DATAFILES
  
  std::string prefix;
  
  if (!quantity.compare("mean")) {
    // construct mean Saver
    CV = &mean_U[0];
    XV = &mean_X[0];
    prefix = "";
    nonnegative = false;
  }
  #if COMPUTE_VARIANCE
  else if (!quantity.compare("variance")) {
    // construct variance Saver
    CV = &var_U[0];
    XV = &var_X[0];
    prefix = "var_";
    if (VARIANCE_FIX)
      nonnegative = true;
    else
      nonnegative = false;
  }
  #endif
  else if (!quantity.compare("debug")) {
    // construct debug Saver
    CV = &mean_U_l[0];
    XV = &mean_X_l[0];
    prefix = "debug_";
    nonnegative = false;
  }
  
  std::ostringstream dfn;
  if (MULTIX > 1 || MULTIY > 1 || MULTIZ > 1)
    dfn << DATA_PATH << prefix << DATA_FILE << "." << multix << "." << multiy << "." << multiz;
  else
    dfn << DATA_PATH << prefix << DATA_FILE;
  data = new std::ofstream(dfn.str().c_str());
  
  #endif
  
  if (multi_main) {
    times = new std::ofstream(TIME_FILE);
    *times << "time = [";
  }
}

Saver::~Saver()
{
  // results in segmentation faults if uncommented!
  //delete data;
  //delete times;
}

template<class T, int G>
void write_data(Saver *s, volume<T,G> &v, int m)
{
    for (int i = 0; i < NX/MULTIX; i++)
      for (int j = 0; j < NY/MULTIY; j++)
        for (int k = 0; k < NZ/MULTIZ; k++)
          s->write( ( (real*)&v[i][j][k] )[m] );
}

void Saver::write(real &r) {
  data -> write ( (char *) &r, sizeof(real) );
}

void Saver::step(volume <ConservedVars, 1> &CV, volume <XtraVars, 1> &XV, real t_halt)
{
  if (multi_samplers_root) {
    if (multi_main) {
      *times << t_halt <<", ";
      times->flush();
    }
    
    #if WRITE_DATAFILES
    
    for (int m=0; m<SIZE_CV; m++)
      write_data (this, CV, m);
    
    for (int m=0; m<SIZE_XV; m++)
      write_data (this, XV, m);
    
    data->flush();
    
    #endif
  }
}

void Saver::snapshots ()
{ 
  for(int s = 0; s<NSAVES+SAVE0; s++) {
    
    #if WRITE_DATAFILES
    
    // for stabilization of ML-part with exp-log
    #if EXP_LOG
    CV[s].log();
    XV[s].log();
    #endif
    
    if (nonnegative) {
      CV[s].nonnegative();
      XV[s].nonnegative();
    }
    
    else {
      // apply generic constraints
      for (int i = 0; i < NXl; i++)
        for (int j = 0; j < NYl; j++)
          for (int k = 0; k < NZl; k++)
            apply_constraints (CV[s] [i][j][k], XV[s] [i][k][k]);
    }
    
    #endif
    
    step(CV[s], XV[s], halt_times[s]);
  }
  
  if (multi_main) {
    *times << "]\n";
    times->flush();
  }
}

#if HISTOGRAMS
void save_hist_data () {
  if (multi_main) {
    std::ostringstream hfn;
    hfn << DATA_PATH << HIST_DATA_FILE;
    std::ofstream *hist_data = new std::ofstream(hfn.str().c_str());
    *hist_data << "hist_data = [ ";
    for (int l=0; l<=L; l++) {
      *hist_data << "[ ";
      for (int i=0; i<HIST_DATA[l].size(); i++)
        *hist_data << HIST_DATA[l][i] << ", ";
      *hist_data << " ], ";
    }
    *hist_data << " ]" << std::endl;
    hist_data->flush();
  }
}
#endif
