
#include "equation.h"
#include "solver-util-hll.h"
#include "solver.h"

///::title:: HLL3 solver ("second" 3 wave) | for Euler
///::depends:: solver-euler-hll-util
///::consts:: HLL_dt=1

void solver_calc_flux(ConservedVars &F, vector &us, vector &Bs,
                      const AllVariables &lV, const AllVariables &rV,
                      int i, int j, int k, int d)
{
  real sl, sr, cs;
  ConservedVars lF, rF, Us;
  
  //hll_speeds(lV, rV, d, sl, sr, cs);
  hll_speeds_ext(lV, rV, d, 1e-5, sl, sr, cs);
  calc_point_flux(lF, lV, d);
  calc_point_flux(rF, rV, d);
  
  real ptl = lV.XU.p;
  real ptr = rV.XU.p;
  
  real udl = lV.XU.u[d]-sl;
  real udr = rV.XU.u[d]-sr;
  
  real aa = udr*rV.U.rho - udl*lV.U.rho;  
  real sm = (rV.U.m[d]*udr - lV.U.m[d]*udl + ptr - ptl) / aa;
  
  us = ( rF.m - lF.m - sr*rV.U.m + sl*lV.U.m ) / aa;
  us[d] = sm;
  
  if (sl == 0) Zero(F);
  else if (sl > 0) {F = lF;}
  else if (sr < 0) {F = rF;}
  else if (sm >= 0) {
    Us.rho = lV.U.rho*udl/(sm-sl);
    Us.m = Us.rho * us;
    real pts = ptl + lV.U.rho*(lV.XU.u[d]-sm)*udl;
    Us.E = (udl*lV.U.E + ptl*lV.XU.u[d] - pts*sm) / (sm-sl);
    F = lF + sl*(Us-lV.U);
  } else {
    Us.rho = rV.U.rho*udr/(sm-sr);
    Us.m = Us.rho * us;
    real pts = ptr + rV.U.rho*(rV.XU.u[d]-sm)*udr;
    Us.E = (udr*rV.U.E + ptr*rV.XU.u[d] - pts*sm) / (sm-sr);
    F = rF + sr*(Us-rV.U);
  }
}
