
#include "equation.h"
#include "space.h"

///::title:: Second Order in Space, Entropy Conservative MINMOD
///::depends:: time_o2 quadrature_o2 powell_o23
///::consts:: ORDER=2 GHOST_SIZE=2
///::vars:: CFL=0.475

real minmod (real a, real b) {
  if (sign(a) == sign(b))
    return sign(a)*min(fabs(a), fabs(b));
  else
    return 0.0;
}

void left(AllVariables &V, int i, int j, int k, int d)
{
  // left and right jumps in ReconstructionVars
  ConservedVars lW = U[i][j][k] - U[i-DX[d]][j-DY[d]][k-DZ[d]];
  ConservedVars rW = U[i+DX[d]][j+DY[d]][k+DZ[d]] - U[i][j][k];
  
  // converting jumps to RoeVars
  ReconstVars_to_ScaledVars (lW, U[i-DX[d]][j-DY[d]][k-DZ[d]], U[i][j][k], Xtra[i-DX[d]][j-DY[d]][k-DZ[d]], Xtra[i][j][k], d);
  ReconstVars_to_ScaledVars (rW, U[i][j][k], U[i+DX[d]][j+DY[d]][k+DZ[d]], Xtra[i][j][k], Xtra[i+DX[d]][j+DY[d]][k+DZ[d]], d);
  
  // computing the slopes in RoeVars and storing in V.U
  for (int m=0; m<SIZE_CV; m++)
    realCast(V.U)[m] = minmod(realCast(lW)[m], realCast(rW)[m])/2.0;
  
  // converting the slopes to ReconstructionVars, using the LEFT converter
  ScaledVars_to_ReconstVars (V.U, U[i-DX[d]][j-DY[d]][k-DZ[d]], U[i][j][k], Xtra[i-DX[d]][j-DY[d]][k-DZ[d]], Xtra[i][j][k], d);
  
  // SUBTRACTING the slope from the cell average U[i][j][k]
  V.U = U[i][j][k] - V.U;
  
  // computing Xtra variables
  calc_xtra(V.XU, V.U, i, j, k);
}

void right(AllVariables &V, int i, int j, int k, int d)
{
  // left and right jumps in ReconstructionVars
  ConservedVars lW = U[i][j][k] - U[i-DX[d]][j-DY[d]][k-DZ[d]];
  ConservedVars rW = U[i+DX[d]][j+DY[d]][k+DZ[d]] - U[i][j][k];
  
  // converting jumps to RoeVars
  ReconstVars_to_ScaledVars (lW, U[i-DX[d]][j-DY[d]][k-DZ[d]], U[i][j][k], Xtra[i-DX[d]][j-DY[d]][k-DZ[d]], Xtra[i][j][k], d);
  ReconstVars_to_ScaledVars (rW, U[i][j][k], U[i+DX[d]][j+DY[d]][k+DZ[d]], Xtra[i][j][k], Xtra[i+DX[d]][j+DY[d]][k+DZ[d]], d);
  
  // computing the slopes in RoeVars and storing in V.U
  for (int m=0; m<SIZE_CV; m++)
    realCast(V.U)[m] = minmod(realCast(lW)[m], realCast(rW)[m])/2.0;
  
  // converting the slopes to ReconstructionVars, using the RIGHT converter
  ScaledVars_to_ReconstVars (V.U, U[i][j][k], U[i+DX[d]][j+DY[d]][k+DZ[d]], Xtra[i][j][k], Xtra[i+DX[d]][j+DY[d]][k+DZ[d]], d);
  
  // ADDING the slope to the cell average U[i][j][k]
  V.U = U[i][j][k] + V.U;
  
  // computing Xtra variables
  calc_xtra(V.XU, V.U, i, j, k);
}
