
#include "space.h"
#include "equation.h"
#include "space-o2eno-util.h"

///::title:: Second Order in Space, positive fixed ENO
///::depends:: time_o2 quadrature_o2 powell_o23 space-o2eno-util
///::consts:: ORDER=2 GHOST_SIZE=2 INDICATOR=1
///::vars:: CFL=0.475
///::macro:: PRE_STEP_HOOK eno.h eno_calc_stencil();

static const real BOUND = 1.9;

void left(AllVariables &V, int i, int j, int k, int d)
{
  PrimitiveVars Pijk(U[i][j][k], Xtra[i][j][k]);
  PrimitiveVars dP;

  if (stencil[d][i][j][k] == -1) {
    dP = Pijk - PrimitiveVars(U[i-DX[d]][j-DY[d]][k-DZ[d]], Xtra[i-DX[d]][j-DY[d]][k-DZ[d]]);
  } else {
    dP = PrimitiveVars(U[i+DX[d]][j+DY[d]][k+DZ[d]], Xtra[i+DX[d]][j+DY[d]][k+DZ[d]]) - Pijk;
  }
  dP.rho = max(-BOUND*Pijk.rho, min(BOUND*Pijk.rho, dP.rho));
  dP.p = max(-BOUND*Pijk.p, min(BOUND*Pijk.p, dP.p));

  real L = 0.125*Pijk.rho*(dP.u.sqr() -
    0.5*min(0, dP.rho * (Pijk.u * dP.u)) +
    0.5*dP.rho*dP.rho*dP.u.sqr()/Pijk.rho;
  
  #if !NO_B
  real LL += 0.125*dP.B.sqr();
  #endif
  
  real R = Pijk.p / (GAMMA-1);
  real aijk = 0.5*sqrt(R/max(R,L));
  //  alpha[d][i][j][k] = aijk;
  //  sdP[d][i][j][k] = dP;

  calc_all_vars(V, Pijk-dP*aijk);
}

void right(AllVariables &V, int i, int j, int k, int d)
{
  PrimitiveVars Pijk(U[i][j][k], Xtra[i][j][k]);
  PrimitiveVars dP;

  if (stencil[d][i][j][k] == -1) {
    dP = Pijk - PrimitiveVars(U[i-DX[d]][j-DY[d]][k-DZ[d]], Xtra[i-DX[d]][j-DY[d]][k-DZ[d]]);
  } else {
    dP = PrimitiveVars(U[i+DX[d]][j+DY[d]][k+DZ[d]], Xtra[i+DX[d]][j+DY[d]][k+DZ[d]]) - Pijk;
  }
  dP.rho = max(-BOUND*Pijk.rho, min(BOUND*Pijk.rho, dP.rho));
  dP.p = max(-BOUND*Pijk.p, min(BOUND*Pijk.p, dP.p));

  real L = 0.125*Pijk.rho*(dP.u.sqr()) -
    0.5*min(0, dP.rho * (Pijk.u * dP.u)) +
    0.5*dP.rho*dP.rho*dP.u.sqr()/Pijk.rho;
  
  #if !NO_B
  real LL += 0.125*dP.B.sqr();
  #endif
  
  real R = Pijk.p / (GAMMA-1);
  real aijk = 0.5*sqrt(R/max(R,L));
  //  alpha[d][i][j][k] = aijk;
  //  sdP[d][i][j][k] = dP;

  calc_all_vars(V, Pijk+dP*aijk);
}
