
#include "equation.h"
#include "space.h"

///::title:: Second Order in Space, plain MINMOD
///::depends:: time_o2 quadrature_o2 powell_o23
///::consts:: ORDER=2 GHOST_SIZE=2
///::vars:: CFL=0.475

real minmod (real a, real b) {
  if (sign(a) == sign(b))
    return sign(a)*min(fabs(a), fabs(b));
  else
    return 0.0;
}

void left(AllVariables &V, int i, int j, int k, int d)
{
  real slopeDX;
  for (int m=0; m<SIZE_CV; m++) {
    slopeDX = minmod(realCast(U[i][j][k])[m] - realCast(U[i-DX[d]][j-DY[d]][k-DZ[d]])[m], realCast(U[i+DX[d]][j+DY[d]][k+DZ[d]])[m] - realCast(U[i][j][k])[m]);
    realCast(V.U)[m] = realCast(U[i][j][k])[m] - slopeDX/2.0;
  }
  calc_xtra(V.XU, V.U, i, j, k);
}

void right(AllVariables &V, int i, int j, int k, int d)
{
  real slopeDX;
  for (int m=0; m<SIZE_CV; m++) {
    slopeDX = minmod(realCast(U[i][j][k])[m] - realCast(U[i-DX[d]][j-DY[d]][k-DZ[d]])[m], realCast(U[i+DX[d]][j+DY[d]][k+DZ[d]])[m] - realCast(U[i][j][k])[m]);
    realCast(V.U)[m] = realCast(U[i][j][k])[m] + slopeDX/2.0;
  }
  calc_xtra(V.XU, V.U, i, j, k);
}
