
/* Buckley-Leverett (scalar nonlinear conservation law)
 * Structures for ConservedVars, XtraVars and PrimitiveVars
 */

#ifndef STRUCTURES_BL_H
#define STRUCTURES_BL_H

#include "structures-base.h"

// === ConservedVars

struct ConservedVars : ConservedVarsBase {
  real   So;
  
  real norm() const
  {
    return fabs(So);
  }
};

// === XtraVars

struct XtraVars : XtraVarsBase {
  real   Sw;
};

// === PrimitiveVars

struct PrimitiveVars : PrimitiveVarsBase {
  real   So;
  
  // converts PrimitiveVars to ConservedVars (CV)
  void toConservedVars(ConservedVars &v) const {
    v.So = So;
  }
  
  inline PrimitiveVars() {}
  // picks PrimitiveVars from ConservedVars (CV) and XtraVars (XV)
  inline PrimitiveVars(const ConservedVars &CV, const XtraVars &XV) {
    So = CV.So;
  }
};

#endif
