
/* Linear Advection (scalar linear conservation law)
 * Structures for ConservedVars, XtraVars and PrimitiveVars
 */

#ifndef STRUCTURES_LINADV_H
#define STRUCTURES_LINADV_H

#include "structures-base.h"

// === ConservedVars

struct ConservedVars : ConservedVarsBase {
  real   u;
  
  real norm() const
  {
    return fabs(u);
  }
};

// === XtraVars

struct XtraVars : XtraVarsBase {
  real   E;
};

// === PrimitiveVars

struct PrimitiveVars : PrimitiveVarsBase {
  real   u;
  
  // converts PrimitiveVars to ConservedVars (CV)
  void toConservedVars(ConservedVars &v) const {
    v.u = u;
  }
  
  inline PrimitiveVars() {}
  // picks PrimitiveVars from ConservedVars (CV) and XtraVars (XV)
  inline PrimitiveVars(const ConservedVars &CV, const XtraVars &XV) {
    u = CV.u;
  }
};

#endif
