
/* Shallow Water equations
 * Structures for ConservedVars, XtraVars and PrimitiveVars
 */

#ifndef STRUCTURES_SW_H
#define STRUCTURES_SW_H

#include "structures-base.h"

// === ConservedVars

struct ConservedVars : ConservedVarsBase {
  real   h;
  vector hu;
  
  real norm() const
  {
    return fabs(h) + hu.norm();
  }
};

// === XtraVars

struct XtraVars : XtraVarsBase {
  vector u;
  real   E;
  real   b;
};

// === PrimitiveVars

struct PrimitiveVars : PrimitiveVarsBase {
  real   h;
  vector u;
  
  // converts PrimitiveVars to ConservedVars (CV)
  void toConservedVars(ConservedVars &v) const {
    v.h = h;
    v.hu = h*u;
  }
  
  inline PrimitiveVars() {}
  // picks PrimitiveVars from ConservedVars (CV) and XtraVars (XV)
  inline PrimitiveVars(const ConservedVars &CV, const XtraVars &XV) {
    h = CV.h;
    u = XV.u;
  }
};

#endif
