
#include "time.h"

#include "equation.h"
#include "multi.h"
#include "space.h"
#include "solver.h"

#if HLL_dt
  #include "solver-util-hll.h"
#endif

void sub_step(bool firsttime)
{
  PRE_STEP_HOOK
  
  // calculate sources
  
  for (int i = 0; i < NXl; i++) {
    for (int j = 0; j < NYl; j++) {
      for (int k = 0; k < NZl; k++) {
        Zero(Source[i][j][k]);
        SOURCE_HOOK
      }
    }
  }
  
  // calculate fluxes
  
  AllVariables lV, rV;
  vector us, Bs;
  
  #if HLL_dt
  vector maxeig;
  maxeig.zero();
  real sl, sr, cs;
  #endif
  
  #if ENTROPIC_RECONSTRUCTION
  ConservedVars_to_ReconstVars();
  #endif
  
  for (int d = 0; d < NDIR; d++) {
    if (!HAVE_DIR(d)) continue;
    for (int i = 0; i < FLUX_NX(d); i++) {
      for (int j = 0; j < FLUX_NY(d); j++) {
        for (int k = 0; k < FLUX_NZ(d); k++) {
          
          right (lV, i-DX[d], j-DY[d], k-DZ[d], d);
          left  (rV, i, j, k, d);
          
          #if HLL_dt
          if (firsttime) {
            hll_speeds(lV, rV, d, sl, sr, cs);
            maxeig[d] = max(maxeig[d],max(fabs(sl),fabs(sr)));
          }
          #endif
          
          solver_calc_flux(Flux[d][i][j][k], us, Bs, lV, rV, i, j, k, d);
          
          SUB_STEP_HOOK
        }
      }
    }
  }
  
  #if ENTROPIC_RECONSTRUCTION
  ReconstVars_to_ConservedVars();
  #endif
  
  // calculate time step
  
  if (firsttime) {
    #if HLL_dt
    dt = multi_dt(CFL/(maxeig[X]/dx[X] + maxeig[Y]/dx[Y] + maxeig[Z]/dx[Z]));
    #else
    dt = multi_dt ( calc_time_step() );
    #endif
    if (multi_main && dt == 0) {
      std::cout << " :: ERROR: time step is 0." << std::endl;
      throw 1;
    }
    if (t + dt > t_halt) dt = t_halt - t;
  }
  
  // apply fluxes & sources
  ConservedVars F;
  for (int i = 0; i < NXl; i++) {
    for (int j = 0; j < NYl; j++) {
      for (int k = 0; k < NZl; k++) {
        for (int d = 0; d < NDIR; d++) {
          if (!HAVE_DIR(d)) continue;
          F = Flux[d][i+DX[d]][j+DY[d]][k+DZ[d]] - Flux[d][i][j][k];
          NON_CONSERVATIVE_FLUX_HOOK
          U[i][j][k] -= (dt/dx[d]) * F;
        }
        U[i][j][k] += dt * Source[i][j][k];
      }
    }
  }
}
