
#include "topography.h"

#include "chaos.h"

///::title:: Karhunen-Loeve expansion (2D version)
///::vars:: KL=3
///::consts:: T_UNIFORM=KL*KL T_NORMAL=0 KL_DECAY=1.5

real b (real x, real y) {
  // Karhunen–Loeve expansion for topography b consisting of KL basis functions
  real value = 0.0;
  for (int mx=1; mx<=KL; mx++)
    for (int my=1; my<=KL; my++)
      value += pow(mx,-KL_DECAY)*pow(my,-KL_DECAY) * sin(mx*pi*x)*sin(my*pi*y) * (2*chaos.t_uniform[(mx-1)*KL+(my-1)] - 1);
  return BED_OFFSET + BED_SCALING * 0.25 * value;
}
